package dsuyasov.recognition.files;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.util.MessageUtil;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.io.IOException;
import java.io.File;
import java.util.Map;
import java.util.HashMap;

/**
 * @author Dmitriy Suyasov
 */
public class SaveFile {

    private static final String DOT = ".";

    private Saver saver;
    private String fileName;

    public SaveFile(Saver saver) {
        this.saver = saver;
        this.fileName = "";
    }

    public String getFileName() {
        return fileName;
    }

    public boolean save() {
        JFileChooser chooser = new JFileChooser();
        RuleFileFilter filter = new RuleFileFilter();
        filter.addExtension(saver.getExtension());
        filter.setDescription(saver.getTitle());
        chooser.setFileFilter(filter);
        int returnVal = chooser.showSaveDialog((JApplet) Memory.get(Memory.APPLET));
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            try {
                fileName = chooser.getSelectedFile().getCanonicalPath();
            } catch (IOException e) {
                MessageUtil.alert(e.getMessage());
                return false;
            }

            if (!fileName.endsWith(DOT + saver.getExtension()))
                fileName += DOT + saver.getExtension();

            return this.saver.save(fileName);
        }
        return false;
    }

    public static class RuleFileFilter extends FileFilter {

        private Map<String, Boolean> extentions = new HashMap<String, Boolean>();
        private String description;

        public boolean accept(File file) {
            if (file.isFile()) {
                int index = file.getName().lastIndexOf(".");
                if (index < 0)
                    return false;
                else
                    return extentions.get(file.getName().substring(index + 1).toLowerCase()) != null;
            } else return true;
        }

        public String getDescription() {
            return description;
        }

        public void setDescription(String description) {
            this.description = description;
        }

        public void addExtension(String s) {
            extentions.put(s, Boolean.TRUE);
        }
    }

}
