package dsuyasov.recognition.rulesModel;

import java.awt.Color;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * Developed by Suyasov Dmitriy
 */
public class MatrixElement {

    private Color color;
    private Color newColor;
    private List<Object> labels;
    private List<Object> removedLabels;
    private List<Object> addedLabels;

    public MatrixElement(Color color, List<Object> labels) {
        this.color = color;
        this.labels = labels;
        this.removedLabels = new LinkedList<Object>();
        this.addedLabels = new LinkedList<Object>();
        this.newColor = color;
    }

    public MatrixElement(Color color, Object... labels) {
        this.color = color;
        this.labels = new ArrayList<Object>();
        for (Object label : labels) {
            this.labels.add(label);
        }
        this.removedLabels = new LinkedList<Object>();
        this.addedLabels = new LinkedList<Object>();
        this.newColor = color;
    }

    public MatrixElement(Color color) {
        this.color = color;
        this.labels = new ArrayList<Object>();
        this.removedLabels = new LinkedList<Object>();
        this.addedLabels = new LinkedList<Object>();
        this.newColor = color;
    }

    public static MatrixElement updateElement(MatrixElement element, Color color) {
        return element == null ? new MatrixElement(color) : element.setColor(color);
    }

    public void makeActions() {
        for (Object removedLabel : removedLabels) {
            labels.remove(removedLabel);
        }
        for (Object addedLabel : addedLabels) {
            labels.add(addedLabel);
        }
        this.color = this.newColor;
        removedLabels.clear();
        addedLabels.clear();
    }

    public Color getColor() {
        return color;
    }

    public List<Object> getLabels() {
        return labels;
    }

    public MatrixElement setColor(Color color) {
        this.newColor = color;
        return this;
    }

    public boolean contain(Object obj) {
        for (Object label : labels) {
            if (label.equals(obj))
                return true;
        }
        return false;
    }

    public void addLabel(Object label) {
        this.addedLabels.add(label);
    }

    public void removeLabel(Object label) {
        for (Iterator<Object> iterator = labels.iterator(); iterator.hasNext();) {
            Object o = iterator.next();
            if (o.equals(label)) {
                removedLabels.add(o);
            }
        }
    }

    public boolean check(Object label) {
        for (Object o : labels) {
            if (o.equals(label))
                return true;
        }
        return false; 
    }
}
