package dsuyasov.recognition.rulesModel;

import dsuyasov.recognition.rulesModel.generators.*;
import dsuyasov.recognition.rulesModel.sequence.MatrixChangeListener;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;

import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * @author Dmitriy Suyasov
 *         Date 15.05.2007
 */
public class RuleCellResult implements XMLTransferable {

    private static final String COLOR_ACCEPTABLE_ATTR_NAME = "colorAcceptable";
    private static final String SYMBOL_GENERATOR_NODE_NAME = "symbolGenerator";
    private static final String SYMBOL_REMOVER_NODE_NAME = "symbolRemover";
    private static final String COLOR_GENERATOR_NODE_NAME = "colorGenerator";

    private LabelGenerator labelGenerator;
    private LabelRemover labelRemover;
    private ColorGenerator colorGenerator;

    private TypeOfColors colorTypeOf;

    public RuleCellResult(TypeOfColors colorTypeOf, RuleStrategy strategy) {
        this.colorTypeOf = colorTypeOf;
        labelGenerator = new DoNothingGenerator();
        labelRemover = new LabelRemover();
        if (strategy.equals(RuleStrategy.NONE) || strategy.equals(RuleStrategy.DO_NOT_CHANGE)) {
            colorGenerator = new NotChangeColorGenerator();
        } else if (strategy.equals(RuleStrategy.ALL_TO_COLOR)) {
            colorGenerator = new FixedColorGenerator();
        } else {
            colorGenerator = new NotChangeColorGenerator();
        }
    }

    public RuleCellResult(Node node, RuleCell centerRuleCell) throws LoadException {
        try {
            colorTypeOf = TypeOfColors.valueOf(XMLUtil.getRequiredAttribute(node, COLOR_ACCEPTABLE_ATTR_NAME));
        } catch (IllegalArgumentException e) {
            throw new LoadException("Incorrect dimention, strategy or the type of the rule was set");
        }
        labelGenerator = LabelGenerator.get(XMLUtil.getSingleNode(node, SYMBOL_GENERATOR_NODE_NAME));
        colorGenerator = ColorGenerator.get(XMLUtil.getSingleNode(node, COLOR_GENERATOR_NODE_NAME));
        labelRemover = new LabelRemover(XMLUtil.getSingleNode(node, SYMBOL_REMOVER_NODE_NAME), centerRuleCell.getLabelCheckers());
    }

    public void initialize(RuleItem ruleItem, MatrixElement[][] source, MatrixChangeListener listener) {
        colorGenerator.initialize(source);
        labelGenerator.initialize(ruleItem, source, listener);
        labelRemover.initialize(source, listener);
    }

    public void process(int x, int y) {
        colorGenerator.setColor(x, y);
        labelGenerator.run(x, y);
        labelRemover.removeLabels(x, y);
    }

    public LabelGenerator getLabelGenerator() {
        return labelGenerator;
    }

    public ColorGenerator getColorGenerator() {
        return colorGenerator;
    }

    public LabelRemover getLabelRemover() {
        return labelRemover;
    }

    public void setSymbolGenerator(LabelGenerator labelGenerator) {
        this.labelGenerator = labelGenerator;
    }

    public void setColorGenerator(ColorGenerator colorGenerator) {
        this.colorGenerator = colorGenerator;
    }

    public TypeOfColors getColorAcceptable() {
        return colorTypeOf;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        getColorGenerator().drawCell(g, rect);
        getLabelGenerator().drawCell(g, rect);
    }

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, COLOR_ACCEPTABLE_ATTR_NAME, colorTypeOf.name());
        labelGenerator.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_GENERATOR_NODE_NAME));
        colorGenerator.toXML(doc, XMLUtil.addNode(doc, rootElement, COLOR_GENERATOR_NODE_NAME));
        labelRemover.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_REMOVER_NODE_NAME));
    }
}
