package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.*;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.ColorCellPanel;
import dsuyasov.recognition.ui.helper.RuleCellChanger;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

import javax.swing.*;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class BetweenColorsChecker extends ColorChecker implements RuleCellChanger {

    private static final String FIRST_COLOR_NODE_NAME = "firstColor";
    private static final String SECOND_COLOR_NODE_NAME = "secondColor";

    private int firstColorR;
    private int firstColorG;
    private int firstColorB;
    private int firstColorA;
    private int secondColorR;
    private int secondColorG;
    private int secondColorB;
    private int secondColorA;

    private static final String FIRST = "first";
    private static final String SECOND = "second";

    public static final String TITLE = "Between colors";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.BETWEEN_COLORS_TITLE);

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public BetweenColorsChecker() {
        setFirstColor(Color.BLACK);
        setSecondColor(Color.WHITE);
    }

    public BetweenColorsChecker(Node node) throws LoadException {
        setFirstColor(XMLUtil.getColor(XMLUtil.getSingleNode(node, FIRST_COLOR_NODE_NAME)));
        setSecondColor(XMLUtil.getColor(XMLUtil.getSingleNode(node, SECOND_COLOR_NODE_NAME)));
    }

    private void setFirstColor(Color color) {
        this.firstColorR = color.getRed();
        this.firstColorG = color.getGreen();
        this.firstColorB = color.getBlue();
        this.firstColorA = color.getAlpha();
    }
    
    private void setSecondColor(Color color) {
        this.secondColorR = color.getRed();
        this.secondColorG = color.getGreen();
        this.secondColorB = color.getBlue();
        this.secondColorA = color.getAlpha();
    }
    
    private Color getFirstColor() {
        return new Color(this.firstColorR, this.firstColorG, this.firstColorB, this.firstColorA);
    }

    private Color getSecondColor() {
        return new Color(this.secondColorR, this.secondColorG, this.secondColorB, this.secondColorA);
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JComponent parent = selecter.getParentPanel();
        JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(mainPanel);
        mainPanel.add(radio);

        TypeOfColors typeOfColors =
                ((CellularAutomata) Memory.get(Memory.CURRENT_RULE)).getDefaultRuleItem().getCell(0,0).getColorAcceptable();
        ColorCellPanel colorPanel1 = new ColorCellPanel(typeOfColors, getFirstColor(), FIRST, this);
        mainPanel.add(colorPanel1);

        ColorCellPanel colorPanel2 = new ColorCellPanel(typeOfColors, getSecondColor(), SECOND, this);
        mainPanel.add(colorPanel2);
        return mainPanel;
    }

    public boolean outOfBoundsChecker() {
        return false;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        Color cellColor = matrix[x][y].getColor();
        int cellColorB = cellColor.getBlue();
        int cellColorG = cellColor.getGreen();
        int cellColorR = cellColor.getRed();
        //int cellColorA = cellColor.getAlpha();
        return cellColorB >= firstColorB && cellColorB <= secondColorB &&
                cellColorR >= firstColorR && cellColorR <= secondColorR &&
                cellColorG >= firstColorG && cellColorG <= secondColorG;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(getFirstColor());
        int[] xPoints = new int[3];
        xPoints[0] = rect.x;
        xPoints[1] = rect.x + rect.width;
        xPoints[2] = rect.x;

        int[] yPoints = new int[3];
        yPoints[0] = rect.y;
        yPoints[1] = rect.y;
        yPoints[2] = rect.y + rect.height;
        g.fillPolygon(xPoints, yPoints, 3);

        g.setColor(getSecondColor());
        xPoints = new int[3];
        xPoints[0] = rect.x;
        xPoints[1] = rect.x + rect.width;
        xPoints[2] = rect.x + rect.width;

        yPoints = new int[3];
        yPoints[0] = rect.y + rect.height;
        yPoints[1] = rect.y + rect.height;
        yPoints[2] = rect.y;
        g.fillPolygon(xPoints, yPoints, 3);
        g.setColor(Color.BLACK);
        g.drawLine(rect.x, rect.y + rect.height, rect.x + rect.width, rect.y);
        g.setColor(Color.BLACK);
        g.drawRect(rect.x, rect.y, rect.width, rect.height);
    }

    public void updateColor(Color color, String label) {
        if (label.equals(FIRST)) {
            setFirstColor(color);
        } else if (label.equals(SECOND)) {
            setSecondColor(color);
        }
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.setColor(doc, XMLUtil.addNode(doc, rootElement, FIRST_COLOR_NODE_NAME), getFirstColor());
        XMLUtil.setColor(doc, XMLUtil.addNode(doc, rootElement, SECOND_COLOR_NODE_NAME), getSecondColor());
    }
}
