package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.ui.helper.Drawer;

import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public abstract class ColorChecker extends Drawer {

    public static List<Class<? extends Drawer>> colorCheckers;

    static {
        colorCheckers = new LinkedList<Class<? extends Drawer>>();
        colorCheckers.add(AnyColorChecker.class);
        colorCheckers.add(FixedColorChecker.class);
        colorCheckers.add(BetweenColorsChecker.class);
        colorCheckers.add(DarkerThenColorCheker.class);
        colorCheckers.add(LighterThenColor.class);
    }

    public static List<Drawer> getDrawers() {
        List<Drawer> drawers = new LinkedList<Drawer>();
        for (Class<? extends Drawer> colorChecker : colorCheckers) {
            try {
                drawers.add(colorChecker.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return drawers;
    }

    public static ColorChecker get(Node node) throws LoadException {
        return (ColorChecker) Drawer.newDrawerInstance(node, getDrawers());
    }

    public boolean check(MatrixElement[][] matrix, int x, int y) {
        if (x < 0 || y < 0 || x >= matrix.length || y >= matrix[0].length)
            return true;
        return internalcheck(matrix, x, y);
    }

    public abstract boolean outOfBoundsChecker();

    public abstract boolean internalcheck(MatrixElement[][] matrix, int x, int y);

}
