package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

/**
 * @author Dmitriy Suyasov
 */
public class NoOfLabelsChecker extends LabelChecker implements LabelsChanges {

    public static final String TITLE = "No of Symbols";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.NO_OF_SYMBOLS_TITLE);

    private static final String SYMBOLS_NODE_NAME = "symbols";
    private static final String SYMBOL_NODE_NAME = "symbol";

    private Selecter selecter;
    private List<Labels> labelsToChecks;

    public NoOfLabelsChecker() {
        this.labelsToChecks = new LinkedList<Labels>();
    }

    public NoOfLabelsChecker(Node node) throws LoadException {
        this.labelsToChecks = new LinkedList<Labels>();
        List<Node> labelsNode = XMLUtil.getNode(XMLUtil.getSingleNode(node, SYMBOLS_NODE_NAME), SYMBOL_NODE_NAME);
        for (Node childNode : labelsNode) {
            labelsToChecks.add(Labels.newLabelInstance(childNode));
        }
    }

    public boolean outOfBoundsChecker() {
        return true;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        for (Labels labels : labelsToChecks) {
            if (matrix[x][y].contain(labels))
                return false;
        }
        return true;
    }

    public Color getColor() {
        return Color.GREEN;
    }

    public String getTitle() {
        return TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JPanel allLabelsCheckerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        selecter.getParentPanel().add(allLabelsCheckerPanel);

        allLabelsCheckerPanel.add(radio);

        this.selecter = selecter;
        JPanel labelsPanel = new SelectLabelsPanel(labelsToChecks, this);
        allLabelsCheckerPanel.add(labelsPanel);
        return allLabelsCheckerPanel;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        g.fillRect(rect.x + rect.width / 2, rect.y + 1, rect.width / 2, rect.height / 2);
        g.setColor(Color.BLACK);
        g.drawLine(rect.x + rect.width * 3 / 5, rect.y + rect.height / 4, rect.x + rect.width - 2, rect.y + rect.height / 4);
    }

    public void updateLabels(List<Labels> symbols) {
        labelsToChecks = new LinkedList<Labels>(symbols);
        selecter.getParentDialog().pack();
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        Node labelsNode = XMLUtil.addNode(doc, rootElement, SYMBOLS_NODE_NAME);
        for (Labels labels : labelsToChecks) {
            labels.toXML(doc, XMLUtil.addNode(doc, labelsNode, SYMBOL_NODE_NAME));
        }
    }
}
