package dsuyasov.recognition.rulesModel.checkers;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.util.List;
import java.util.LinkedList;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class OneOfTheLabelsChecker extends LabelChecker implements LabelsChanges {

    public static final String TITLE = "One of the Symbols";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.ONE_OF_THE_SYMBOLS_TITLE);

    private List<Labels> labelsToChecks;
    private Selecter selecter;

    private static final String SYMBOLS_NODE_NAME = "symbols";
    private static final String SYMBOL_NODE_NAME = "symbol";

    public OneOfTheLabelsChecker() {
        this.labelsToChecks = new LinkedList<Labels>();
    }
    
    public OneOfTheLabelsChecker(Node node) throws LoadException {
        this.labelsToChecks = new LinkedList<Labels>();
        List<Node> labelsNode = XMLUtil.getNode(XMLUtil.getSingleNode(node, SYMBOLS_NODE_NAME), SYMBOL_NODE_NAME);
        for (Node childNode : labelsNode) {
            labelsToChecks.add(Labels.newLabelInstance(childNode));
        }
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public boolean outOfBoundsChecker() {
        return false;
    }

    public boolean internalcheck(MatrixElement[][] matrix, int x, int y) {
        for (Object obj : labelsToChecks) {
            for (Object c : matrix[x][y].getLabels()) {
                if (obj.equals(c)) {
                    addLabelToLabelsRemover(c);
                    return true;
                }
            }
        }
        return false;
    }

    public Color getColor() {
        return Color.RED;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JPanel allLabelsCheckerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        selecter.getParentPanel().add(allLabelsCheckerPanel);

        allLabelsCheckerPanel.add(radio);

        this.selecter = selecter;
        JPanel labelsPanel = new SelectLabelsPanel(labelsToChecks, this);
        allLabelsCheckerPanel.add(labelsPanel);
        return allLabelsCheckerPanel;
    }

    public void updateLabels(List<Labels> labels) {
        labelsToChecks = new LinkedList<Labels>(labels);
        this.selecter.getParentDialog().pack();
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        g.fillRect(rect.x + rect.width / 2, rect.y + 1, rect.width / 2, rect.height / 2);
        g.setColor(Color.BLACK);
        int[] x = new int[3];
        x[0] = rect.x + rect.width * 3 / 5;
        x[1] = rect.x + rect.width * 3 / 5;
        x[2] = rect.x + rect.width - 2;
        int[] y = new int[3];
        y[0] = rect.y + 2;
        y[1] = rect.y + rect.height * 2 / 5 + 1;
        y[2] = rect.y + rect.height * 2 / 5 + 1;
        g.drawPolygon(x, y, 3);
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        Node labelsNode = XMLUtil.addNode(doc, rootElement, SYMBOLS_NODE_NAME);
        for (Labels labels : labelsToChecks) {
            labels.toXML(doc, XMLUtil.addNode(doc, labelsNode, SYMBOL_NODE_NAME));
        }
    }
}