package dsuyasov.recognition.rulesModel.generators;

import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class AddLabelGenerator extends LabelGenerator implements LabelsChanges {

    private List<Labels> labels;
    private Selecter selecter;

    private static final String SYMBOL_NODE_NAME = "symbol";

    public AddLabelGenerator() {
        this.labels = new LinkedList<Labels>();
    }

    public AddLabelGenerator(Node node) throws LoadException {
        List<Node> nodes = XMLUtil.getNode(node, SYMBOL_NODE_NAME);
        this.labels = new LinkedList<Labels>();
        for (Node node1 : nodes) {
            this.labels.add(Labels.newLabelInstance(node1));
        }
    }

    public static final String TITLE = "Add Symbol";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.ADD_SYMBOL_TITLE);

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        this.selecter = selecter;
        JComponent parent = selecter.getParentPanel();
        JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(mainPanel);
        mainPanel.add(radio);

        JPanel labelsPanel = new SelectLabelsPanel(labels, this);
        mainPanel.add(labelsPanel);
        return mainPanel;
    }

    public void updateLabels(List<Labels> labels) {
        this.labels = new LinkedList<Labels>(labels);
        this.selecter.getParentDialog().pack();
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        g.fillRect(rect.x + rect.width / 2, rect.y + 1, rect.width / 2, rect.height / 2);
        g.setColor(Color.BLACK);
        g.drawLine(rect.x + rect.width * 3 / 4, rect.y, rect.x + rect.width * 3 / 4, rect.y + rect.height / 2);
        g.drawLine(rect.x + rect.width / 2, rect.y + rect.height / 4, rect.x + rect.width , rect.y + rect.height / 4);
    }

    protected void generateNewLabels(int x, int y) {
        for (Labels label : labels) {
            matrix[x][y].addLabel(label);
        }
        setChange(true);
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        for (Labels label : this.labels) {
            label.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_NODE_NAME));
        }
    }

}
