package dsuyasov.recognition.rulesModel.generators;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.ui.helper.Drawer;

import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public abstract class ColorGenerator extends Drawer {

    public static List<Class<? extends Drawer>> colorGenerators;

    private boolean change;

    static {
        colorGenerators = new LinkedList<Class<? extends Drawer>>();
        colorGenerators.add(NotChangeColorGenerator.class);
        colorGenerators.add(FixedColorGenerator.class);
        colorGenerators.add(MiddleGreyColorGenerator.class);
    }

    public static List<Drawer> getDrawers() {
        List<Drawer> drawers = new LinkedList<Drawer>();
        for (Class<? extends Drawer> colorGenerator : colorGenerators) {
            try {
                drawers.add(colorGenerator.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return drawers;
    }

    public boolean isChange() {
        return change;
    }

    public void setChange(boolean change) {
        this.change = change;
    }

    protected MatrixElement[][] matrix;

    public void initialize(MatrixElement[][] matrix) {
        this.matrix = matrix;
    }

    public abstract void setColor(int x, int y);

    public static ColorGenerator get(Node node) throws LoadException {
        return (ColorGenerator) Drawer.newDrawerInstance(node, getDrawers());
    }
}
