package dsuyasov.recognition.rulesModel.generators;

import dsuyasov.recognition.rulesModel.RuleItem;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.sequence.MatrixChangeListener;
import dsuyasov.recognition.ui.helper.Drawer;

import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public abstract class LabelGenerator extends Drawer {

    public static final String GENERATOR_NODE_NAME = "generator";

    public static List<Class<? extends Drawer>> symbolGenerators;

    private MatrixChangeListener changerListener;

    static {
        symbolGenerators = new LinkedList<Class<? extends Drawer>>();
        symbolGenerators.add(DoNothingGenerator.class);
        symbolGenerators.add(AddLabelGenerator.class);
        symbolGenerators.add(IncreaseGenerator.class);
        symbolGenerators.add(MinimumOfGenerated.class);
    }

    public static List<Drawer> getDrawers() {
        List<Drawer> drawers = new LinkedList<Drawer>();
        for (Class<? extends Drawer> symbolGenerator : symbolGenerators) {
            try {
                drawers.add(symbolGenerator.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return drawers;
    }

    public void setChange(boolean change) {
        if (change) changerListener.change();
    }

    public static LabelGenerator get(Node node) throws LoadException {
        return (LabelGenerator) Drawer.newDrawerInstance(node, getDrawers());
    }

    protected RuleItem ruleItem;
    protected MatrixElement[][] matrix;

    public void initialize(RuleItem ruleItem, MatrixElement[][] matrix, MatrixChangeListener listener) {
        this.ruleItem = ruleItem;
        this.matrix = matrix;
        this.changerListener = listener;
    }

    public void run(int x, int y) {
        generate(x, y);
    }

    protected void generate(int x, int y) {
        generateNewLabels(x, y);
    }
    protected abstract void generateNewLabels(int x, int y);

}
