package dsuyasov.recognition.rulesModel.generators;

import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.Selecter;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Developed by Suyasov Dmitriy
 */
public class MinimumOfGenerated extends LabelGenerator {

    private static final String CROSS_NODE_NAME = "cross";
    private static final String VALUE_ATTR_NAME = "value";

    private boolean cross;

    public MinimumOfGenerated() {
        this.cross = false;
    }

    public MinimumOfGenerated(Node node) throws LoadException {
        String attribute = XMLUtil.getAttribute(XMLUtil.getSingleNode(node, CROSS_NODE_NAME), VALUE_ATTR_NAME);
        this.cross = attribute != null && Boolean.parseBoolean(attribute);
    }

    public static final String TITLE = "Minimum of Generated";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.MINIMUM_OF_GENERATED_TITLE);

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public JPanel drawElement(Selecter selecter, JComponent radio) {
        JComponent parent = selecter.getParentPanel();
        JPanel mainPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(mainPanel);
        mainPanel.add(radio);

        JCheckBox crossCheckBox = new JCheckBox(Localization.get(Localization.CROSS_BUTTON_NAME), this.cross);
        crossCheckBox.addChangeListener(new CrossCheckboxSelected());
        mainPanel.add(crossCheckBox);
        return mainPanel;
    }

    public void drawCell(Graphics g, Rectangle rect) {
        g.setColor(Color.WHITE);
        g.fillRect(rect.x + rect.width / 2, rect.y + 1, rect.width / 2, rect.height / 2);
        g.setColor(Color.BLACK);
        g.drawLine(rect.x + rect.width * 3 / 4, rect.y + 2, rect.x + rect.width * 3 / 4, rect.y + rect.height / 2);
        g.drawLine(rect.x + rect.width * 3 / 4, rect.y + rect.height / 2, rect.x + rect.width * 3 / 4 + 3, rect.y + rect.height / 4);
        g.drawLine(rect.x + rect.width * 3 / 4, rect.y + rect.height / 2, rect.x + rect.width * 3 / 4 - 3, rect.y + rect.height / 4);
    }

    protected void generateNewLabels(int x, int y) {
        int dimention = ruleItem.getDimention();
        int xBorder = Math.min(x + dimention, matrix.length - 1);
        int yBorder = Math.min(y + dimention, matrix[0].length - 1);
        int minNumber = Integer.MAX_VALUE;
        Object minSymbol = null;
        for (Object label : matrix[x][y].getLabels()) {
            if (label instanceof Integer) {
                minNumber = Integer.parseInt(label.toString());
                minSymbol = label;
            }
        }
        int sMinNumber = minNumber;
        setChange(false);
        for (int iLoop1 = Math.max(x - dimention, 0); iLoop1 <= xBorder; iLoop1++) {
            for (int iLoop2 = Math.max(y - dimention, 0); iLoop2 <= yBorder; iLoop2++) {
                if (cross || x == iLoop1 && y != iLoop2 || y == iLoop2 && x != iLoop1) {
                    for (Object label : matrix[iLoop1][iLoop2].getLabels()) {
                        if (label instanceof Integer) {
                            int oldMin = minNumber;
                            minNumber = Math.min((Integer) label, minNumber);
                            if (oldMin != minNumber) {
                                setChange(true);
                            }
                        }
                    }
                }
            }
        }
        if (minNumber < sMinNumber) {
            matrix[x][y].removeLabel(minSymbol);
            matrix[x][y].addLabel(minNumber);
            matrix[x][y].makeActions();
        }
    }


    private class CrossCheckboxSelected implements ChangeListener {

        public void stateChanged(ChangeEvent e) {
            cross = ((JCheckBox) e.getSource()).isSelected();
        }
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, XMLUtil.addNode(doc, rootElement, CROSS_NODE_NAME), VALUE_ATTR_NAME, String.valueOf(cross));
    }
}
