package dsuyasov.recognition.rulesModel.labels;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;

/**
 * @author Dmitriy Suyasov
 */
public class CustomLabels extends Labels {

    public static final String CUSTOM_SYMBOLS_TITLE = "Custom Symbol";

    private static final String TEXT_NODE_NAME = "text";
    private static final String VALUE_ATTR_NAME = "value";

    private String text;

    public CustomLabels() {
        this.text = "";
    }

    public CustomLabels(Node node) throws LoadException {
        this.text = XMLUtil.getRequiredAttribute(XMLUtil.getSingleNode(node, TEXT_NODE_NAME), VALUE_ATTR_NAME);
    }

    public String getTitle() {
        return CUSTOM_SYMBOLS_TITLE;
    }

    public String getLocalizationTitle() {
        return Localization.get(Localization.CUSTOM_SYMBOL_TITLE);
    }

    public void draw(Graphics g, int x, int y, int width, int height) {
        g.setColor(Color.BLACK);
        g.fillRect(x + width / 4, y + height / 4, width / 2, height / 2);
        g.setColor(Color.WHITE);
        g.drawRect(x + width / 4, y + height / 4, width / 2, height / 2);
    }

    public void drawDialog(JComponent parent) {
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parent.add(panel);
        panel.add(new JLabel(Localization.get(Localization.CUSTOM_SYMBOL_TITLE)));
        JTextField textField = new JTextField(this.text, 5);
        textField.addActionListener(new CustomLabelsListener(this));
        panel.add(textField);
    }

    public boolean verify() {
        return this.text != null && this.text.length() > 0 && this.text.length() < 4;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getText() {
        return text;
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, XMLUtil.addNode(doc, rootElement, TEXT_NODE_NAME), VALUE_ATTR_NAME, getText());
    }

    private class CustomLabelsListener implements ActionListener {

        private CustomLabels customLabels;

        public CustomLabelsListener(CustomLabels customLabels) {
            this.customLabels = customLabels;
        }

        public void actionPerformed(ActionEvent e) {
            this.customLabels.setText(((JTextField) e.getSource()).getText());
        }
    }


    public boolean equals(Object obj) {
        if (!(obj instanceof CustomLabels))
            return false;
        CustomLabels customLabels = (CustomLabels) obj;
        return this.getText().equals(customLabels.getText());
    }
}
