package dsuyasov.recognition.rulesModel.labels;

import dsuyasov.recognition.files.XMLTransferable;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.util.XMLUtil;

import javax.swing.*;
import java.awt.Graphics;
import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * @author Dmitriy Suyasov
 */
public abstract class Labels implements XMLTransferable {

    private static final String TYPE_ATTR_NAME = "type";

    public static List<Class<? extends Labels>> labels;

    static {
        labels = new LinkedList<Class<? extends Labels>>();
        labels.add(PredefinedLabels.class);
        labels.add(CustomLabels.class);
    }

    public static List<Labels> getLabels() {
        List<Labels> allLabels = new LinkedList<Labels>();
        for (Class<? extends Labels> symbol : labels) {
            try {
                allLabels.add(symbol.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return allLabels;
    }

    public static Labels newLabelInstance(Node node) throws LoadException {
        String type = XMLUtil.getRequiredAttribute(node, TYPE_ATTR_NAME);
        List<Labels> symbols = getLabels();
        Labels newSymbol = null;
        for (Labels symbol : symbols) {
            if (symbol.getTitle().equals(type)) {
                try {
                    newSymbol = symbol.getClass().getConstructor(Node.class).newInstance(node);
                } catch (Exception e) {
                    throw new LoadException("Problem while initializing Label");
                }
            }
        }
        if (newSymbol == null)
            throw new LoadException("Label type [" + type + "] is unknown");
        return newSymbol;
    }

    public abstract String getTitle();
    public abstract String getLocalizationTitle();
    public abstract void draw(Graphics g, int x, int y, int width, int height);
    public abstract void drawDialog(JComponent parent);
    public abstract boolean verify();

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, TYPE_ATTR_NAME, getTitle());
    }

}
