package dsuyasov.recognition.rulesModel.sequence;

import dsuyasov.recognition.ui.helper.panels.AutomataSequencePanel;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.ProcessResult;
import dsuyasov.recognition.rulesModel.CellularAutomata;

import javax.swing.*;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * @author Dmitriy Suyasov
 *         Date 17.05.2007
 */
public class AnyChangeConditionElement extends SpecialElement{

    public static final String TITLE = "Any change condition";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.ANY_CHANGE_CONDITION_TITLE);

    public static final String SELECTED_LABEL_INDEX_ATTR_NAME = "selectedLabelIndex";

    private List<LabelElement> labels;
    private int selectedLabelIndex;
    private List<Integer> labelsIndexes;
    private AutomataSequence automataSequence;

    private int loopIndex;

    public AnyChangeConditionElement() {
        labels = new LinkedList<LabelElement>();
        labelsIndexes = new LinkedList<Integer>();
        selectedLabelIndex = -1;
    }

    public AnyChangeConditionElement(AnyChangeConditionElement anyChangeConditionElement, AutomataSequence automataSequence) {
        labels = anyChangeConditionElement.getLabels();
        labelsIndexes = anyChangeConditionElement.getLabelsIndexes();
        selectedLabelIndex = anyChangeConditionElement.getSelectedLabelIndex();
        this.automataSequence = automataSequence;
        loopIndex = anyChangeConditionElement.getLoopIndex();
    }

    public AnyChangeConditionElement(Node node) throws LoadException {
        this.selectedLabelIndex = XMLUtil.getIntAttribute(node, SELECTED_LABEL_INDEX_ATTR_NAME);
    }

    public List<LabelElement> getLabels() {
        return labels;
    }

    public int getLoopIndex() {
        return loopIndex;
    }

    public List<Integer> getLabelsIndexes() {
        return labelsIndexes;
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        if (selectedLabelIndex < 0) {
            g.setColor(Color.RED);
        } else {
            g.setColor(Color.BLACK);
        }
        int rectEnd = rect.x + rect.width;
        int[] xx = new int[9];
        xx[0] = rect.x;
        xx[1] = rect.x + rect.width / 5;
        xx[2] = rect.x + rect.width * 4 / 5;
        xx[3] = rectEnd;
        xx[4] = rectEnd;
        xx[5] = rect.x + rect.width * 4 / 5;
        xx[6] = rect.x + rect.width / 5;
        xx[7] = rect.x;
        xx[8] = rect.x;
        int[] yy = new int[9];
        yy[0] = rect.y + rect.height * 4 / 5;
        yy[1] = rect.y + rect.height;
        yy[2] = rect.y + rect.height;
        yy[3] = rect.y + rect.height * 4 / 5;
        yy[4] = rect.y + rect.height / 5;
        yy[5] = rect.y;
        yy[6] = rect.y;
        yy[7] = rect.y + rect.height / 5;
        yy[8] = rect.y + rect.height * 4 / 5;
        g.drawPolyline(xx, yy, 9);

        if (selectedLabelIndex >= 0) {
            int middle = rect.height / 2;
            int rectEndAndOffset = rectEnd + (labels.size() - selectedLabelIndex) * 10;
            int rectMiddle = rect.y + middle;
            int middleOfLabel = labelsIndexes.get(selectedLabelIndex) * AutomataSequencePanel.RULES_STEP +
                    AutomataSequencePanel.Y_RULES_START_POINT + middle;
            g.drawLine(rectEnd, rectMiddle, rectEndAndOffset, rectMiddle);
            g.drawLine(rectEndAndOffset, rectMiddle, rectEndAndOffset, middleOfLabel);
            g.drawLine(rectEndAndOffset, middleOfLabel, rectEnd, middleOfLabel);
            g.drawLine(rectEnd, middleOfLabel, rectEnd + 7, middleOfLabel - 5);
            g.drawLine(rectEnd, middleOfLabel, rectEnd + 7, middleOfLabel + 5);
        }

        super.draw(g, rect, rect.width / 10, yTextOffset);
    }

    public String getDescription() {
        return Localization.get(Localization.ANY_CHANGE_CONDITION_DESCRIPTION);
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        JPanel linkLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(linkLabelPanel);

        if (selectedLabelIndex < 0)
            selectedLabelIndex = labels.isEmpty() ? -1 : 0;
        String[] labelTitles = new String[labels.size()];
        for (int iLoop = 0; iLoop < labels.size(); iLoop++) {
            LabelElement labelElement = labels.get(iLoop);
            labelTitles[iLoop] = labelElement.getLocalizationTitle() + " " + labelsIndexes.get(iLoop);
        }

        linkLabelPanel.add(new JLabel(Localization.get(Localization.LABEL_OF_TRANSITION_TITLE)));
        JComboBox labelsCombo = new JComboBox(labelTitles);
        labelsCombo.setSelectedIndex(selectedLabelIndex);
        labelsCombo.addActionListener(new ChangeLabelSelectionListener());
        linkLabelPanel.add(labelsCombo);
    }

    public int getSelectedLabelIndex() {
        return selectedLabelIndex;
    }

    public int getSelectedIndex() {
        return labelsIndexes.get(selectedLabelIndex);
    }

    public boolean confirmCorrectness() {
        return selectedLabelIndex >= 0;
    }

    public String getErrorMessage() {
        return (selectedLabelIndex < 0) ? Localization.get(Localization.SELECT_LABEL_FIRST_MESSAGE) : "";
    }

    public boolean canBeRemoved() {
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {
        labels = new LinkedList<LabelElement>();
        labelsIndexes = new LinkedList<Integer>();
        for (int iLoop = 0; iLoop < automataSequence.getRules().size(); iLoop++) {
            SequenceDisplayable sequenceDisplayable = automataSequence.getRules().get(iLoop);
            if (sequenceDisplayable instanceof LabelElement) {
                labels.add((LabelElement) sequenceDisplayable);
                labelsIndexes.add(iLoop);
            }
        }
    }

    public void updateInformationRulesMoved(AutomataSequence automataSequence, int indexOfFirst) {
        List<Processable> rules = automataSequence.getRules();
        Processable firstEl = rules.get(indexOfFirst);
        Processable secondEl = rules.get(indexOfFirst + 1);
        if ((firstEl instanceof LabelElement) && (secondEl instanceof LabelElement)) {
            if (labelsIndexes.get(selectedLabelIndex) == indexOfFirst) selectedLabelIndex++;
            if (labelsIndexes.get(selectedLabelIndex) == indexOfFirst + 1) selectedLabelIndex--;
        }
        labels.clear();
        labelsIndexes.clear();
        for (int iLoop = 0; iLoop < rules.size(); iLoop++) {
            Processable rule = rules.get(iLoop);
            if (rule instanceof LabelElement) {
                labels.add((LabelElement) rule);
                labelsIndexes.add(iLoop);
            }
        }
    }

    public void updateInformationRuleRemoved(AutomataSequence automataSequence, int index) {
        int number = 0;
        List<Processable> rules = automataSequence.getRules();
        for (Processable processable : rules) {
            if (processable instanceof LabelElement) {
                number++;
            }
        }
        if (number != labels.size() && labelsIndexes.get(selectedLabelIndex) >= index) {
            selectedLabelIndex--;
        }
        labels.clear();
        labelsIndexes.clear();
        for (int iLoop = 0; iLoop < rules.size(); iLoop++) {
            Processable rule = rules.get(iLoop);
            if (rule instanceof LabelElement) {
                labels.add((LabelElement) rule);
                labelsIndexes.add(iLoop);
            }
        }
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, rootElement, SELECTED_LABEL_INDEX_ATTR_NAME, String.valueOf(selectedLabelIndex));
    }

    public void initialize(AutomataSequence automataSequence) {
        this.automataSequence = automataSequence;
        this.loopIndex = 0;

        Integer labelIndex = labelsIndexes.get(selectedLabelIndex);
        for (int iLoop = 0; iLoop < automataSequence.getRules().size(); iLoop++) {
            Processable ruleProcessable =  automataSequence.getRules().get(iLoop);
            if (ruleProcessable == this) {
                for (int iLoop2 = labelIndex + 1; iLoop2 < iLoop; iLoop2++) {
                    Processable processable =  automataSequence.getRules().get(iLoop2);
                    if (processable instanceof CellularAutomata) {
                        ((CellularAutomata) processable).addChangeListener((LabelElement) automataSequence.getRules().get(labelIndex));
                    }
                }
            }
        }
    }

    public Integer getNext(int ruleIndex) {
        if (((LabelElement) this.automataSequence.getRules().get(labelsIndexes.get(selectedLabelIndex).intValue())).isChange()) {
            if (loopIndex > 100) {
                this.loopIndex = 0;
                return super.getNext(ruleIndex);
            }
            this.loopIndex++;
            return labelsIndexes.get(selectedLabelIndex);
        } else {
            this.loopIndex = 0;
            return super.getNext(ruleIndex);
        }
    }

    public ProcessResult process(MatrixElement[][] source) {

        return new ProcessResult(source);
    }

    private class ChangeLabelSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JComboBox combo = (JComboBox) e.getSource();
            selectedLabelIndex = combo.getSelectedIndex();
        }
    }
}
