package dsuyasov.recognition.rulesModel.sequence;

import dsuyasov.recognition.rulesModel.ProcessResult;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.FlowLayout;
import java.util.List;
import java.util.LinkedList;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

/**
 * @author Dmitriy Suyasov
 *         Date 24.05.2007
 */
public class GetMetric extends SpecialElement implements LabelsChanges {

    public static final String TITLE = "Get metric";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.GET_METRIC_TITLE);

    private static final String SYMBOL_NODE_NAME = "symbol";

    private List<Labels> labels;
    private JDialog dialog;

    public GetMetric() {
        this.labels = new LinkedList<Labels>();
    }

    public GetMetric(Node node) throws LoadException {
        this.labels = new LinkedList<Labels>();
        List<Node> labelNodes = XMLUtil.getNode(node, SYMBOL_NODE_NAME);
        for (Node labelNode : labelNodes) {
            this.labels.add(Labels.newLabelInstance(labelNode));
        }
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public String getDescription() {
        return "Get Metric"; // TODO make description
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        this.dialog = dialog;
        JPanel symbolsSettingsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(symbolsSettingsPanel);

        symbolsSettingsPanel.add(new JLabel("Labels to display"));
        JPanel symbolsPanel = new SelectLabelsPanel(labels, this);
        symbolsSettingsPanel.add(symbolsPanel);
    }

    public boolean confirmCorrectness() {
        return labels != null && labels.size() > 0;
    }

    public String getErrorMessage() {
        return "At least one symbol should be selected";
    }

    public boolean canBeRemoved() {
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {
    }

    public void initialize(AutomataSequence automataSequence) {
    }

    public ProcessResult process(MatrixElement[][] source) {
        return null;
    }

    public void updateLabels(List<Labels> symbols) {
        this.labels = new LinkedList<Labels>(symbols);
        dialog.pack();
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        for (Labels symbol : labels) {
            symbol.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_NODE_NAME));
        }
    }
}
