package dsuyasov.recognition.rulesModel.sequence;

import dsuyasov.recognition.rulesModel.ProcessResult;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.ui.helper.panels.DrawImagePanel;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.panels.AutomataSequencePanel;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.LinkedList;
import java.util.Hashtable;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

/**
 * @author Dmitriy Suyasov
 */
public class OutputerElement extends SpecialElement implements LabelsChanges {

    public static final String TITLE = "Outputter";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.OUTPUTTER_TITLE);

    private JPanel picturePanel;
    private JList resultList;
    private DefaultListModel resultListModel;

    //private List<DrawImagePanel> images;

    public static final String SYMBOL_NODE_NAME = "symbol";

    private JDialog dialog;

    private List<Labels> labels;

    public OutputerElement() {
        labels = new LinkedList<Labels>();
    }

    public void addLabel(Labels label) {
        this.labels.add(label);
    }

    public OutputerElement(Node node) throws LoadException {
        this.labels = new LinkedList<Labels>();
        List<Node> labelNodes = XMLUtil.getNode(node, SYMBOL_NODE_NAME);
        for (Node labelNode : labelNodes) {
            this.labels.add(Labels.newLabelInstance(labelNode));
        }
    }

    public OutputerElement(OutputerElement outputerElement, AutomataSequence automataSequence) {
        this.labels = outputerElement.getLabels();
    }

    public List<Labels> getLabels() {
        return labels;
    }

    private AutomataSequence automataSequence;

    public void initialize(JPanel parentPanel, AutomataSequence automataSequence) {
        parentPanel.removeAll();
        parentPanel.setLayout(new BoxLayout(parentPanel, BoxLayout.Y_AXIS));

        this.automataSequence = automataSequence;

        JPanel buttonsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        parentPanel.add(buttonsPanel);

        drawStartButton(buttonsPanel);

        JPanel mainListPanel = new JPanel(new BorderLayout(5, 5));
        parentPanel.add(mainListPanel);

        AutomataSequencePanel automataSequencePanel = new AutomataSequencePanel(automataSequence, true);

        automataSequencePanel.scrollRectToVisible(automataSequencePanel.getSizes());
        automataSequencePanel.setPreferredSize(automataSequencePanel.getSizes().getSize());
        JScrollPane ruleSequenceScrollPane = new JScrollPane(automataSequencePanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        ruleSequenceScrollPane.setPreferredSize(new Dimension(280, 370));
        ruleSequenceScrollPane.setMinimumSize(new Dimension(280, 370));
        mainListPanel.add(ruleSequenceScrollPane, BorderLayout.WEST);

        JPanel listPanel = new JPanel(new BorderLayout(5, 5));
        mainListPanel.add(listPanel, BorderLayout.EAST);

        listPanel.add(new JLabel(Localization.get(Localization.RESULTS_LIST_TITLE) + ":"), BorderLayout.NORTH);

        resultListModel = new DefaultListModel();
        resultList = new JList(resultListModel);
        resultList.addListSelectionListener(new ChangeResultListSelection(new LinkedList<DrawImagePanel>()));
        resultList.setBorder(BorderFactory.createEtchedBorder());
        resultList.setVisibleRowCount(-1);
        resultList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        resultList.setLayoutOrientation(JList.VERTICAL);
        JScrollPane pane = new JScrollPane(resultList);
        pane.setPreferredSize(new Dimension(170, 410));
        pane.setMinimumSize(new Dimension(170, 410));
        listPanel.add(pane, BorderLayout.CENTER);

        resultList.scrollRectToVisible(new Rectangle(140, 400));
        resultList.revalidate();
        resultList.repaint();
        
        picturePanel = new JPanel();
        mainListPanel.add(picturePanel, BorderLayout.CENTER);
    }

    private void initializeRecognition() {
        clearMatrixAndImage();
        this.automataSequence.initializeProcessing(PictureToMatrixElementsSpecialElement.convert((BufferedImage) Memory.get(Memory.PICTURE)));
    }

    private void drawButton(JPanel parent, String name, ActionListener listener) {
        JButton button = new JButton(name);
        button.addActionListener(listener);
        parent.add(button);
    }

    private void drawStartButton(JPanel buttonsPanel) {
        buttonsPanel.removeAll();
        drawButton(buttonsPanel, Localization.get(Localization.START_DIALOG_BUTTON_NAME), new StartOrBackListener(buttonsPanel, false));
    }

    private void drawRunButtons(JPanel buttonsPanel) {
        buttonsPanel.removeAll();
        drawButton(buttonsPanel, Localization.get(Localization.BACK_TO_BEGIN_DIALOG_BUTTON_NAME), new StartOrBackListener(buttonsPanel, true));
        drawButton(buttonsPanel, Localization.get(Localization.RUN_DIALOG_BUTTON_NAME), new RunListener(buttonsPanel, RunType.ALL));
        drawButton(buttonsPanel, Localization.get(Localization.RUN_AUTOMAT_DIALOG_BUTTON_NAME), new RunListener(buttonsPanel, RunType.AUTOMAT));
        drawButton(buttonsPanel, Localization.get(Localization.RUN_STEP_DIALOG_BUTTON_NAME), new RunListener(buttonsPanel, RunType.STEP));
    }

    private void checkButtons(JPanel buttonsPanel) {
        if (((List) Memory.get(Memory.PROCESSING_ELEMENTS)).isEmpty()) {
            buttonsPanel.removeAll();
            drawButton(buttonsPanel, Localization.get(Localization.BACK_TO_BEGIN_DIALOG_BUTTON_NAME), new StartOrBackListener(buttonsPanel, true));
            MessageUtil.setMessage(Localization.get(Localization.CELL_AUTOMAT_PROCESSED_TITLE, Memory.getIncreaser().toString()));
        }
    }

    private void fillResultList() {
        List<DrawImagePanel> drawImagePanels = new LinkedList<DrawImagePanel>();
        int savedSelectedIndex = resultList.getSelectedIndex();
        resultListModel.clear();
        for (DrawImagePanel drawImagePanel : (List<DrawImagePanel>) Memory.get(Memory.IMAGES)) {
            resultListModel.add(resultListModel.size(), Localization.get(Localization.ELEMENT_TITLE) + " " + (resultListModel.size() + 1));
        }
        for (AutomataSequence.ForProcess forProcess : ((List<AutomataSequence.ForProcess>) Memory.get(Memory.PROCESSING_ELEMENTS))) {
            resultListModel.add(resultListModel.size(), Localization.get(Localization.ELEMENT_TITLE) + " " + (resultListModel.size() + 1));
            drawImagePanels.add(new DrawImagePanel(forProcess.getElements(), labels));
        }
        List<Integer> scales = new LinkedList<Integer>();
        for (ListSelectionListener listSelectionListener : resultList.getListSelectionListeners()) {
            resultList.removeListSelectionListener(listSelectionListener);
            for (DrawImagePanel drawImagePanel : ((ChangeResultListSelection) listSelectionListener).getPredefinedPanels()) {
                scales.add(drawImagePanel.getScale());
            }
        }
        for (int iLoop = 0; iLoop < drawImagePanels.size(); iLoop++) {
            DrawImagePanel drawImagePanel = drawImagePanels.get(iLoop);
            drawImagePanel.setScale(scales.size() == 0 ? 1 : scales.get(Math.min(scales.size() - 1, iLoop)));
        }
        resultList.addListSelectionListener(new ChangeResultListSelection(drawImagePanels));
        resultList.setSelectedIndex(savedSelectedIndex);

        resultList.scrollRectToVisible(new Rectangle(160, 400));
        resultList.revalidate();
        resultList.repaint();
    }

    private enum RunType {
        STEP, AUTOMAT, ALL
    }

    private class RunListener implements ActionListener {
        private JPanel buttonsPanel;
        private RunType runType;

        public RunListener(JPanel buttonsPanel, RunType runType) {
            this.buttonsPanel = buttonsPanel;
            this.runType = runType;
        }

        public void actionPerformed(ActionEvent e) {
            if (runType.equals(RunType.STEP)) {
                automataSequence.processStep();
            } else if (runType.equals(RunType.AUTOMAT)) {
                automataSequence.processAutomata();
            } else {
                automataSequence.processAll();
            }

            fillResultList();

            // Update button panel
            checkButtons(buttonsPanel);

            ((JPanel) Memory.get(Memory.MAIN_PANEL)).revalidate();
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
        }
    }

    private class StartOrBackListener implements ActionListener {
        private JPanel buttonsPanel;
        private boolean back;

        public StartOrBackListener(JPanel buttonsPanel, boolean back) {
            this.buttonsPanel = buttonsPanel;
            this.back = back;
        }

        public void actionPerformed(ActionEvent e) {
            if (back) {
                drawStartButton(buttonsPanel);
            } else {
                drawRunButtons(buttonsPanel);
                initializeRecognition();
                fillResultList();
            }
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).revalidate();
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
        }
    }


    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public String getDescription() {
        return Localization.get(Localization.DISPLAYING_DESCRIPTION);
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        this.dialog = dialog;
        JPanel labelsSettingsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(labelsSettingsPanel);

        labelsSettingsPanel.add(new JLabel(Localization.get(Localization.LABELS_TO_DISPLAY_TITLE)));
        JPanel labelsPanel = new SelectLabelsPanel(labels, this);
        labelsSettingsPanel.add(labelsPanel);
    }

    public boolean confirmCorrectness() {
        return true;
    }

    public String getErrorMessage() {
        return "";
    }

    public boolean canBeRemoved() {
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {
        // SYSTEM ELEMENT: no neen in this
    }

    public void initialize(AutomataSequence automataSequence) {
    }

    private void clearMatrixAndImage() {
        Memory.add(Memory.MATRIX, new LinkedList<MatrixElement[][]>());
        Memory.add(Memory.IMAGES, new LinkedList<DrawImagePanel>());
    }

    private void saveMatrix(MatrixElement[][] matrix) {
        List<MatrixElement[][]> elements = (List<MatrixElement[][]>) Memory.get(Memory.MATRIX);
        if (elements == null) {
            elements = new LinkedList<MatrixElement[][]>();
            Memory.add(Memory.MATRIX, elements);
        }
        elements.add(matrix);
    }

    private void saveImages(DrawImagePanel image) {
        List<DrawImagePanel> elements = (List<DrawImagePanel>) Memory.get(Memory.IMAGES);
        if (elements == null) {
            elements = new LinkedList<DrawImagePanel>();
            Memory.add(Memory.IMAGES, elements);
        }
        elements.add(image);
    }

    public ProcessResult process(MatrixElement[][] source) {
        saveMatrix(source);
        saveImages(new DrawImagePanel(source, labels));
        return new ProcessResult();
    }

    public void updateLabels(List<Labels> symbols) {
        this.labels = new LinkedList<Labels>(symbols);
        dialog.pack();
    }

    private class ChangeResultListSelection implements ListSelectionListener {

        private List<DrawImagePanel> predefinedPanels;

        public ChangeResultListSelection(List<DrawImagePanel> predefinedPanels) {
            this.predefinedPanels = predefinedPanels;
        }

        public List<DrawImagePanel> getPredefinedPanels() {
            return predefinedPanels;
        }

        public void valueChanged(ListSelectionEvent e) {
            JList list = (JList) e.getSource();
            List<DrawImagePanel> savedImages = (List<DrawImagePanel>) Memory.get(Memory.IMAGES);
            int selectedIndex = list.getSelectedIndex();
            if (selectedIndex < 0) {
                picturePanel.removeAll();
                repaint();
            } else {
                if (selectedIndex >= savedImages.size()) {
                    selectedIndex -= savedImages.size();
                    drawImage(predefinedPanels.get(selectedIndex));
                } else {
                    drawImage(savedImages.get(selectedIndex));
                }
            }
        }

        private void drawImage(DrawImagePanel image) {
            picturePanel.removeAll();
            picturePanel.setLayout(new BorderLayout(5, 5));

            JScrollPane scrollPanel = new JScrollPane();
            scrollPanel.setPreferredSize(new Dimension(300, 400));
            scrollPanel.setMinimumSize(new Dimension(300, 400));
            Rectangle rect = new Rectangle(image.getWidth(), image.getHeight());
            image.scrollRectToVisible(rect);
            Dimension dim = new Dimension(image.getWidth(), image.getHeight());
            image.setPreferredSize(dim);
            scrollPanel.setViewportView(image);

            picturePanel.add(scrollPanel, BorderLayout.CENTER);

            JSlider scaleSlider = new JSlider(JSlider.VERTICAL, 1, 100, image.getScale());
            scaleSlider.addChangeListener(new SliderListener(image));
            scaleSlider.setMajorTickSpacing(10);
            scaleSlider.setPaintTicks(true);
            picturePanel.add(scaleSlider, BorderLayout.EAST);

            //Create the label table
            Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
            labelTable.put( 1, new JLabel("x1") );
            labelTable.put( 50, new JLabel("x50") );
            labelTable.put( 100, new JLabel("x100") );
            scaleSlider.setLabelTable( labelTable );

            scaleSlider.setPaintLabels(true);

            repaint();
        }

        private void repaint() {
            picturePanel.revalidate();
            picturePanel.repaint();
        }
    }

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        for (Labels symbol : labels) {
            symbol.toXML(doc, XMLUtil.addNode(doc, rootElement, SYMBOL_NODE_NAME));
        }
    }

    private class SliderListener implements ChangeListener {

        private DrawImagePanel drawImagePanel;

        public SliderListener(DrawImagePanel drawImagePanel) {
            this.drawImagePanel = drawImagePanel;
        }

        public void stateChanged(ChangeEvent e) {

            drawImagePanel.setScale(((JSlider)e.getSource()).getValue());

            Rectangle rect = new Rectangle(drawImagePanel.getWidth(), drawImagePanel.getHeight());
            Dimension dim = new Dimension(drawImagePanel.getWidth(), drawImagePanel.getHeight());

            drawImagePanel.setSize(dim);
            drawImagePanel.scrollRectToVisible(rect);
            drawImagePanel.setPreferredSize(dim);

            drawImagePanel.revalidate();
            drawImagePanel.repaint();
        }
    }
}
