package dsuyasov.recognition.rulesModel.sequence;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.swing.*;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.LinkedList;
import java.util.List;

import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.ProcessResult;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.util.XMLUtil;

/**
 * @author Dmitriy Suyasov
 *         Date 17.05.2007
 */
public abstract class SpecialElement extends Processable {

    public static final String TYPE_ATTR_NAME = "type";

    public static List<Class<? extends SpecialElement>> specialElements;

    static {
        specialElements = new LinkedList<Class<? extends SpecialElement>>();
        specialElements.add(LabelElement.class);
        specialElements.add(AnyChangeConditionElement.class);
        //specialElements.add(UnitCells.class); TODO Test and make it available
        specialElements.add(SplitIntoLabelsElement.class);
        specialElements.add(MarkTopLeftElement.class);
        specialElements.add(OutputerElement.class);
    }

    public static List<SpecialElement> getSpecialElements() {
        List<SpecialElement> specialElementsInstances = new LinkedList<SpecialElement>();
        for (Class<? extends SpecialElement> specialElementClass: specialElements) {
            try {
                specialElementsInstances.add(specialElementClass.newInstance());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return specialElementsInstances;
    }

    public boolean isSpecialElement() {
        return true;
    }

    public boolean isRule() {
        return false;
    }

    public abstract String getTitle();
    
    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        g.drawString(getLocalizationTitle(), rect.x + xTextOffset, rect.y + yTextOffset);
    }

    public void drawSettings(JDialog dialog, JPanel parentPanel, AutomataSequence automataSequence) {
        parentPanel.removeAll();
        updateInformation(automataSequence);
        drawSettingsFromCleanList(dialog, parentPanel, automataSequence);
    }

    protected abstract void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence);

    public abstract boolean confirmCorrectness();
    public abstract String getErrorMessage();

    public abstract boolean canBeRemoved();

    public abstract void updateInformation(AutomataSequence automataSequence);
    public void updateInformationRulesMoved(AutomataSequence automataSequence, int indexOfFirst) {}
    public void updateInformationRuleRemoved(AutomataSequence automataSequence, int index) {}

    public static SpecialElement newSpecialElementInstance(Node node) throws LoadException {
        String type = XMLUtil.getRequiredAttribute(node, TYPE_ATTR_NAME);
        SpecialElement newSpecialElement = null;
        for (SpecialElement specialElement : getSpecialElements()) {
            if (specialElement.getTitle().equals(type)) {
                try {
                    newSpecialElement = specialElement.getClass().getConstructor(Node.class).newInstance(node);
                } catch (Exception e) {
                    throw new LoadException("Problem while initializing Special Element");
                }
            }
        }
        if (newSpecialElement == null)
            throw new LoadException("Special Element type [" + type + "] is unknown");
        return newSpecialElement;
    }

    public void toXML(Document doc, Node rootElement) {
        XMLUtil.addAttribute(doc, rootElement, TYPE_ATTR_NAME, getTitle());
    }

    public static SpecialElement newInstance(SpecialElement specialElement, AutomataSequence automataSequence) {
        if (specialElement instanceof OutputerElement)
            return specialElement;
        for (Class<? extends SpecialElement> specialElementClass : specialElements) {
            if (specialElementClass.equals(specialElement.getClass())) {
                try {
                    return specialElementClass.getConstructor(specialElementClass, AutomataSequence.class).newInstance(specialElement, automataSequence);
                } catch (Exception e) {
                    throw new RuntimeException("Exception while creating new instance of SpecialElement");
                }
            }
        }
        return null;
    }

    public ProcessResult processStep(MatrixElement[][] source) {
        return process(source);
    }
}