package dsuyasov.recognition.rulesModel.sequence;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import dsuyasov.recognition.util.XMLUtil;
import dsuyasov.recognition.util.localization.Localization;
import dsuyasov.recognition.files.LoadException;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.ProcessResult;

/**
 * @author Dmitriy Suyasov
 *         Date 18.05.2007
 */
public class UnitCells extends SpecialElement{

    public static final String TITLE = "Unit Cells";
    public static final String LOCALIZATION_TITLE = Localization.get(Localization.UNIT_CELLS_TITLE);

    public static final String TYPE_OF_UNITING_ATTR_NAME = "typeOfUniting";
    public static final String DIVIDE_TIMES_ATTR_NAME = "divideTimes";
    public static final String CELLS_NUMBER_ATTR_NAME = "cellsNumber";

    public static final String TYPE_UNITING = "unating";
    public static final String TYPE_DIVIDING = "dividing";

    private int cellsNumber;
    private int divideTimes;
    private String typeOfUniting;

    private JTextField cellsNumberField;
    private JTextField divideNumberField;

    public UnitCells() {
        cellsNumber = 2;
        divideTimes = 40;
        typeOfUniting = TYPE_DIVIDING; 
    }

    public UnitCells(Node node) throws LoadException {
        typeOfUniting = XMLUtil.getAttribute(node, TYPE_OF_UNITING_ATTR_NAME);
        if (!typeOfUniting.equals(TYPE_UNITING) && !typeOfUniting.equals(TYPE_DIVIDING)) {
            throw new LoadException("Incorrect Type Of Dividing for Uniting Cells");
        }
        divideTimes = XMLUtil.getIntAttribute(node, DIVIDE_TIMES_ATTR_NAME);
        cellsNumber = XMLUtil.getIntAttribute(node, CELLS_NUMBER_ATTR_NAME);
    }

    public UnitCells(UnitCells unitCells, AutomataSequence automataSequence) {
        cellsNumber = unitCells.getCellsNumber();
        divideTimes = unitCells.getDivideTimes();
        typeOfUniting = unitCells.getTypeOfUniting();
    }

    public int getCellsNumber() {
        return cellsNumber;
    }

    public int getDivideTimes() {
        return divideTimes;
    }

    public String getTypeOfUniting() {
        return typeOfUniting;
    }

    public String getTitle() {
        return TITLE;
    }

    public String getLocalizationTitle() {
        return LOCALIZATION_TITLE;
    }

    public void draw(Graphics g, Rectangle rect, int xTextOffset, int yTextOffset) {
        g.setColor(Color.BLACK);
        g.drawRect(rect.x + 4, rect.y, rect.width - 8, rect.height);
        g.drawRect(rect.x, rect.y + 4, 4, rect.height - 8);
        g.drawRect(rect.x + rect.width - 4, rect.y + 4, 4, rect.height - 8);
        super.draw(g, rect, rect.x + 8, yTextOffset);
    }

    public String getDescription() {
        return "Uniting"; // TODO make description
    }

    protected void drawSettingsFromCleanList(JDialog dialog, JPanel panel, AutomataSequence automataSequence) {
        ButtonGroup group = new ButtonGroup();

        JPanel unitSomeCellsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(unitSomeCellsPanel);

        JRadioButton unitCellsRadio = new JRadioButton("Unit ");
        unitCellsRadio.addActionListener(new SelectRadioListener(TYPE_UNITING));
        unitSomeCellsPanel.add(unitCellsRadio);
        group.add(unitCellsRadio);
        cellsNumberField = new JTextField(String.valueOf(this.cellsNumber));
        cellsNumberField.setColumns(6);
        unitSomeCellsPanel.add(cellsNumberField);
        unitSomeCellsPanel.add(new JLabel("cells"));

        JPanel dividePicturePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panel.add(dividePicturePanel);

        JRadioButton divideMatrixRadio = new JRadioButton("Divide matrix into ");
        divideMatrixRadio.addActionListener(new SelectRadioListener(TYPE_DIVIDING));
        dividePicturePanel.add(divideMatrixRadio);
        group.add(divideMatrixRadio);
        divideNumberField = new JTextField(String.valueOf(this.divideTimes));
        divideNumberField.setColumns(6);
        dividePicturePanel.add(divideNumberField);
        dividePicturePanel.add(new JLabel("cells"));

        if (typeOfUniting.equals(TYPE_DIVIDING))
            divideMatrixRadio.setSelected(true);
        else
            unitCellsRadio.setSelected(true);
    }

    public void initialize(AutomataSequence automataSequence) {
    }

    public ProcessResult process(MatrixElement[][] source) {
        return new ProcessResult(source);
    }

    private class SelectRadioListener implements ActionListener {

        private String type;

        public SelectRadioListener(String type) {
            this.type = type;
        }

        public void actionPerformed(ActionEvent e) {
            if (((JRadioButton) e.getSource()).isSelected()) {
                typeOfUniting = type;
            }
        }
    }

    public boolean confirmCorrectness() {
        if (typeOfUniting.equals(TYPE_DIVIDING)) {
            try {
                divideTimes = Integer.parseInt(divideNumberField.getText());
            } catch(NumberFormatException nfe) {
                return false;
            }
        } else {
            try {
                cellsNumber = Integer.parseInt(cellsNumberField.getText());
            } catch(NumberFormatException nfe) {
                return false;
            }
        }
        return true;
    }

    public String getErrorMessage() {
        return "Please correct " + (typeOfUniting.equals(TYPE_DIVIDING) ? "dividing number" : "cells number");
    }

    public boolean canBeRemoved() {
        return true;
    }

    public void updateInformation(AutomataSequence automataSequence) {}

    public void toXML(Document doc, Node rootElement) {
        super.toXML(doc, rootElement);
        XMLUtil.addAttribute(doc, rootElement, TYPE_OF_UNITING_ATTR_NAME, typeOfUniting);
        XMLUtil.addAttribute(doc, rootElement, DIVIDE_TIMES_ATTR_NAME, String.valueOf(divideTimes));
        XMLUtil.addAttribute(doc, rootElement, CELLS_NUMBER_ATTR_NAME, String.valueOf(cellsNumber));
    }
}
