package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.rulesModel.ContinueCondition;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

/**
 * @author Dmitriy Suyasov
 *         Date 16.04.2007
 */
public class AutomataSettingsDialog extends AbstractDialog implements KeyListener {

    public static final String TITLE = Localization.get(Localization.RULE_SETTINGS_DIALOG_TITLE);

    private JTextField ruleNameBox;
    private JTextArea  ruleDescription;
    private JComboBox continueCondition;

    public AutomataSettingsDialog() throws HeadlessException {
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);

        // Add CellularAutomata Name
        JPanel ruleNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        mainPanel.add(ruleNamePanel);
        JLabel ruleNameLabel = new JLabel(Localization.get(Localization.RULE_NAME_FIELD_TITLE));
        ruleNamePanel.add(ruleNameLabel);
        ruleNameBox = new JTextField(cellularAutomata.getName());
        ruleNameBox.setPreferredSize(new Dimension(200, 20));
        ruleNamePanel.add(ruleNameBox);

        // CellularAutomata description
        JPanel descLabelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        descLabelPanel.add(new JLabel(Localization.get(Localization.DESCRIPTION_FIELD_TITLE)));
        mainPanel.add(descLabelPanel);
        JPanel descAreaPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        ruleDescription = new JTextArea(cellularAutomata.getDescription(), 8, 30);
        ruleDescription.setLineWrap(true);
        JScrollPane descScroll = new JScrollPane(ruleDescription);
        descAreaPanel.add(descScroll);
        mainPanel.add(descAreaPanel);
        ruleDescription.revalidate();
        ruleDescription.repaint();

        // CellularAutomata description
        JPanel continueCondPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        mainPanel.add(continueCondPanel);
        continueCondPanel.add(new JLabel("Continue condition:"));
        continueCondition = new JComboBox(ContinueCondition.getTitles());
        continueCondition.addKeyListener(this);
        continueCondition.setSelectedItem(cellularAutomata.getContinueCondition().getTitle());
        continueCondPanel.add(continueCondition);

        drawUpdateCancelButtons(mainPanel);
        
        setContentPane(mainPanel);
        setResizable(false);
        addKeyListener(this);
    }

    public void updateAndClose() {
        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        cellularAutomata.setName(ruleNameBox.getText());
        cellularAutomata.setDescription(ruleDescription.getText());
        cellularAutomata.setContinueCondition(ContinueCondition.getContinueCondition(continueCondition.getSelectedItem().toString()));
        close();
    }

    public void keyTyped(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            updateAndClose();
        } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
            close();
        }
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
    }

    public String getDialogTitle() {
        return TITLE;
    }
}
