package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.sequence.SpecialElement;
import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Dmitriy Suyasov
 *         Date 18.05.2007
 */
public class ChooseSpecialRuleDialog extends AbstractDialog {

    public static final String TITLE = Localization.get(Localization.SPECIAL_ELEMENT_CHOOSER_DIALOG_TITLE);

    public static final int DIALOG_WIDTH = 300;
    public static final int DIALOG_HEIGHT = 400;

    private Integer selectedElementIndex;
    private List<SpecialElement> specElementList;
    private JComboBox specRulesCombo;
    private JPanel specRuleSettingsPanel;

    public ChooseSpecialRuleDialog() throws HeadlessException {
        specElementList = SpecialElement.getSpecialElements();
        initializeDialog(false);
    }

    public ChooseSpecialRuleDialog(SpecialElement specialElement) throws HeadlessException {
        specElementList = new LinkedList<SpecialElement>();
        specElementList.add(specialElement);
        initializeDialog(true);
    }

    public String getDialogTitle() {
        return TITLE;
    }

    private void initializeDialog(boolean update) {
        selectedElementIndex = 0;

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        setContentPane(mainPanel);

        if (specElementList.size() > 1) {
            JPanel specRuleSelecterPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 10));
            mainPanel.add(specRuleSelecterPanel);
            specRuleSelecterPanel.add(new JLabel(Localization.get(Localization.SPECIAL_RULE_FIELD_TITLE)));
            specRulesCombo = new JComboBox(getSpecialElementTitles());
            specRulesCombo.addActionListener(new SpecRulesComboListener(this));
            specRuleSelecterPanel.add(specRulesCombo);
        }

        specRuleSettingsPanel = new JPanel();
        specRuleSettingsPanel.setLayout(new BoxLayout(specRuleSettingsPanel, BoxLayout.Y_AXIS));
        mainPanel.add(specRuleSettingsPanel);
        specElementList.get(0).drawSettings(this, specRuleSettingsPanel, AutomataSequence.getRulesSequence());

        if (update)
            drawUpdateCancelButtons(mainPanel);
        else
            drawOkCancelButtons(mainPanel);

        pack();
        setVisible(true);
    }

    private String[] getSpecialElementTitles() {
        String[] result = new String[specElementList.size()];
        for (int iLoop = 0; iLoop < specElementList.size(); iLoop++) {
            SpecialElement specialElement = specElementList.get(iLoop);
            result[iLoop] = specialElement.getLocalizationTitle();
        }
        return result;
    }

    public void updateAndClose() {
        if (specRulesCombo != null) {
            selectedElementIndex = specRulesCombo.getSelectedIndex();
            if (!specElementList.get(selectedElementIndex).confirmCorrectness()) {
                MessageUtil.alert(specElementList.get(selectedElementIndex).getErrorMessage());
            } else {
                setVisible(false);
            }
        } else {
            setVisible(false);
        }
    }

    public void close() {
        selectedElementIndex = null;
        super.close();
    }

    public SpecialElement getSelectedElement() {
        return selectedElementIndex == null ? null : specElementList.get(selectedElementIndex);
    }

    private class SpecRulesComboListener implements ActionListener {

        private JDialog dialog;

        public SpecRulesComboListener(JDialog dialog) {
            this.dialog = dialog;
        }

        public void actionPerformed(ActionEvent e) {
            int newSelection = specRulesCombo.getSelectedIndex();
            if (selectedElementIndex != newSelection) {
                selectedElementIndex = newSelection;
                specElementList.get(newSelection).drawSettings(dialog, specRuleSettingsPanel, AutomataSequence.getRulesSequence());
                pack();
                repaint();
            }
        }
    }

}
