package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.*;
import dsuyasov.recognition.files.*;
import dsuyasov.recognition.ui.page.ShowAutomataPage;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author Dmitriy Suyasov
 *         Date 12.04.2007
 */
public class NewAutomataSettingsDialog extends AbstractDialog implements KeyListener {

    public static final String TITLE = Localization.get(Localization.NEW_RULE_SETTINGS_DIALOG_TITLE);

    public static final int WIDTH = 300;
    public static final int HEIGHT = 460;

    private JTextField ruleName;
    private JComboBox ruleDimention;
    private JComboBox ruleStrategy;
    private JComboBox continueCondition;
    private JComboBox colors;
    private JComboBox resultColors;

    //private ColorCellPanel strategyResultColor = null;
    //private Map<String, Object> settingsMap;

    public NewAutomataSettingsDialog() throws HeadlessException {
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        JPanel elementsPanel = new JPanel();
        elementsPanel.setLayout(new BoxLayout(elementsPanel, BoxLayout.Y_AXIS));
        mainPanel.add(elementsPanel);

        // Common settings
        JPanel commonSettingsPanel = new JPanel();
        commonSettingsPanel.setLayout(new BoxLayout(commonSettingsPanel, BoxLayout.Y_AXIS));
        commonSettingsPanel.setBorder(BorderFactory.createTitledBorder(Localization.get(Localization.COMMON_SETTINGS_TITLE)));
        elementsPanel.add(commonSettingsPanel);
        JPanel namePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        commonSettingsPanel.add(namePanel);
        namePanel.add(new JLabel(Localization.get(Localization.RULE_NAME_FIELD_TITLE) + ": "));
        ruleName = new JTextField("");
        ruleName.setColumns(20);
        ruleName.addKeyListener(this);
        namePanel.add(ruleName);
        JPanel dimentionsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        commonSettingsPanel.add(dimentionsPanel);
        dimentionsPanel.add(new JLabel(Localization.get(Localization.RULE_DIMENTION_FIELD_TITLE)));
        ruleDimention = new JComboBox(new String[] {AutomataDimention.DIMENTION_1_CELL, AutomataDimention.DIMENTION_9_CELLS, AutomataDimention.DIMENTION_25_CELLS});
        ruleDimention.addKeyListener(this);
        dimentionsPanel.add(ruleDimention);
        JPanel colorAcceptancePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        commonSettingsPanel.add(colorAcceptancePanel);
        colorAcceptancePanel.add(new JLabel(Localization.get(Localization.COLORS_FIELD_TITLE)));
        colors = new JComboBox(new String[] {TypeOfColors.BLACK_AND_WHITE_COLORS, TypeOfColors.RGB_COLORS});
        colors.addKeyListener(this);
        colorAcceptancePanel.add(colors);
        JPanel resultColorAcceptancePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        commonSettingsPanel.add(resultColorAcceptancePanel);
        resultColorAcceptancePanel.add(new JLabel(Localization.get(Localization.RESULT_COLORS_FIELD_TITLE)));
        resultColors = new JComboBox(new String[] {TypeOfColors.BLACK_AND_WHITE_COLORS, TypeOfColors.RGB_COLORS});
        resultColors.addKeyListener(this);
        resultColorAcceptancePanel.add(resultColors);

        // Initial settings
        JPanel initialSettingsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        initialSettingsPanel.setBorder(BorderFactory.createTitledBorder(Localization.get(Localization.INITIAL_SETTINGS_TITLE)));
        elementsPanel.add(initialSettingsPanel);
        initialSettingsPanel.add(new JLabel(Localization.get(Localization.STRATEGY_FIELD_TITLE)));
        ruleStrategy = new JComboBox(new String[] {RuleStrategy.STRATEGY_DO_NOT_CHANGE, RuleStrategy.STRATEGY_ALL_TO_COLOR});
        ruleStrategy.addKeyListener(this);
        initialSettingsPanel.add(ruleStrategy);

        // Continue condition settings
        JPanel continueConditionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        continueConditionPanel.setBorder(BorderFactory.createTitledBorder(Localization.get(Localization.CONTINUE_CONDITION_TITLE)));
        elementsPanel.add(continueConditionPanel);
        continueConditionPanel.add(new JLabel(Localization.get(Localization.CONDITION_FIELD_TITLE)));
        continueCondition = new JComboBox(ContinueCondition.getTitles());
        continueCondition.addKeyListener(this);
        continueConditionPanel.add(continueCondition);

        drawOkCancelButtons(mainPanel, Localization.get(Localization.CREATE_DIALOG_BUTTON_NAME), Localization.get(Localization.CANCEL_DIALOG_BUTTON_NAME));
        
        mainPanel.setPreferredSize(new Dimension(WIDTH, HEIGHT));
        setContentPane(mainPanel);
    }

    public void clearAndHide() {
        setVisible(false);
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            updateAndClose();
        } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
            close();
        }
    }

    public void updateAndClose() {
        close();
        String name = ruleName.getText();
        String dimention = (String) ruleDimention.getSelectedItem();
        String strategy = (String) ruleStrategy.getSelectedItem();
        String continueCondition = (String) this.continueCondition.getSelectedItem();
        String colorsToBeAccepted = (String) colors.getSelectedItem();
        String colorsForResultsToBeAccepted = (String) resultColors.getSelectedItem();

        Memory.add(Memory.SELECTED_RULE, 0);
        Memory.add(Memory.CURRENT_RULE, new CellularAutomata(name, RuleStrategy.getRuleStrategy(strategy),
                AutomataDimention.getRuleDimention(dimention), ContinueCondition.getContinueCondition(continueCondition),
                TypeOfColors.getColors(colorsToBeAccepted), TypeOfColors.getColors(colorsForResultsToBeAccepted)));
        MessageUtil.setMessage(Localization.get(Localization.RULE_CREATED_MESSAGE));
        ShowAutomataPage.setAsCurrent();
    }

    public String getDialogTitle() {
        return TITLE;
    }

    private class CancelActionListener implements ActionListener {

        private NewAutomataSettingsDialog parent;

        public CancelActionListener(NewAutomataSettingsDialog parent) {
            this.parent = parent;
        }

        public void actionPerformed(ActionEvent e) {
            parent.setVisible(false);
        }

    }

    private class CreateActionListener implements ActionListener {

        private NewAutomataSettingsDialog parent;

        public CreateActionListener(NewAutomataSettingsDialog parent) {
            this.parent = parent;
        }

        public void actionPerformed(ActionEvent e) {
            if (Memory.get(Memory.CURRENT_RULE) != null) {
                int result = JOptionPane.showConfirmDialog(parent, Localization.get(Localization.SAVE_BEFORE_CREATING_QUESTION), Localization.get(Localization.WARNING_DIALOG_TITLE), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
                if (result == JOptionPane.CANCEL_OPTION) {
                    return;
                }
                if (result == JOptionPane.YES_OPTION) {
                    if (!new SaveFile(new dsuyasov.recognition.files.CellularAutomataSaver((CellularAutomata) Memory.get(Memory.CURRENT_RULE))).save())
                        return;
                }
            }
            parent.updateAndClose();
        }
    }

}
