package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.recognition.StudiesSet;
import dsuyasov.recognition.rulesModel.recognition.AnalizeResult;
import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.files.LoadFile;
import dsuyasov.recognition.files.TemplateLoader;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.ui.helper.panels.SelectLabelsPanel;
import dsuyasov.recognition.ui.helper.panels.DrawImagePanel;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.util.*;
import java.util.List;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author Dmitriy Suyasov
 */
public class ResolveDialog extends AbstractDialog implements LabelsChanges {

    public static final String TITLE = Localization.get(Localization.RESOLVING_DIALOG_TITLE);

    private List<Labels> symbols;
    private JPanel mainPanel;

    public ResolveDialog() throws HeadlessException {
        mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        symbols = new LinkedList<Labels>();
        if (Memory.get(Memory.LAYERS_LABELS) != null) {
            symbols.addAll((List) Memory.get(Memory.LAYERS_LABELS));
        }
        JPanel forSymbolsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        forSymbolsPanel.add(new JLabel(Localization.get(Localization.SYMBOLS_TO_ACCEPT_FOR_LAYERS_FIELD_TITLE)));
        JPanel symbolsPanel = new SelectLabelsPanel(symbols, this, Memory.LAYERS_LABELS);
        forSymbolsPanel.add(symbolsPanel);
        mainPanel.add(forSymbolsPanel);

        JPanel choosePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        mainPanel.add(choosePanel);
        if (Memory.get(Memory.TEMPLATE) != null) {
            JRadioButton useInMemoryTemplate = new JRadioButton(Localization.get(Localization.USE_SAVED_TEMPLATE_BUTTON_NAME));
            choosePanel.add(useInMemoryTemplate);
            choosePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
            JRadioButton useInLoadedTemplate = new JRadioButton(Localization.get(Localization.USE_LOADED_TEMPLATE_BUTTON_NAME));
            choosePanel.add(useInLoadedTemplate);
            mainPanel.add(choosePanel);
        }
        JTextField templateFileName = new JTextField(getTemplateFileName(), 20);
        choosePanel.add(templateFileName);
        JButton loadTemplate = new JButton(Localization.get(Localization.LOAD_TEMPLATE_BUTTON_NAME));
        loadTemplate.addActionListener(new LoadTemplateListener(templateFileName));
        choosePanel.add(loadTemplate);

        JPanel resultPanel = new JPanel();
        resultPanel.setLayout(new BoxLayout(resultPanel, BoxLayout.Y_AXIS));

        JPanel startButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        JButton startButton = new JButton(Localization.get(Localization.START_RECOGNIZING_BUTTON_NAME));
        startButton.addActionListener(new StartListener(resultPanel));
        startButtonPanel.add(startButton);
        mainPanel.add(startButtonPanel);

        mainPanel.add(resultPanel);

        if (Memory.get(Memory.SAVED_RESULTS) != null)
            drawResults(resultPanel);

        setContentPane(mainPanel);
        setResizable(false);
        mainPanel.revalidate();
        mainPanel.repaint();
        pack();
        setVisible(true);
    }

    public String getDialogTitle() {
        return TITLE;
    }

    protected void updateAndClose() {}

    private class StartListener implements ActionListener {

        private JPanel resultPanel;

        public StartListener(JPanel resultPanel) {
            this.resultPanel = resultPanel;
        }

        public void actionPerformed(ActionEvent e) {
            if (symbols.size() == 0) {
                MessageUtil.alert(Localization.get(Localization.SELECT_SYMBOLS_ALERT));
                return;
            }
            if (Memory.get(Memory.TEMPLATE) == null) {
                MessageUtil.alert(Localization.get(Localization.LOAD_TEMPLATE_ALERT));
                return;
            }
            resultPanel.removeAll();

            StringBuffer resultText = new StringBuffer();

            List<List<AnalizeResult>> results = new LinkedList<List<AnalizeResult>>();
            StudiesSet studiesSet = (StudiesSet) Memory.get(Memory.TEMPLATE);
            List<MatrixElement[][]> elements = (List<MatrixElement[][]>) Memory.get(Memory.MATRIX);
            String[] listTitles = new String[elements.size()];
            int index = 0;
            for (MatrixElement[][] element : elements) {
                List<AnalizeResult> analizeResults = studiesSet.analize(element, symbols);
                String el = "";
                if (analizeResults == null || analizeResults.size() == 0) {
                    el = "-";
                } else {
                    el = analizeResults.get(0).getLabel();
                }
                resultText.append(el);
                listTitles[index++] = el;
                Collections.sort(analizeResults, new AnalizeResult.ByLabelComparator());
                results.add(analizeResults);
            }

            Memory.add(Memory.SAVED_RESULTS, new SavedResults(results, resultText.toString(), listTitles));
            drawResults(resultPanel);
        }
    }

    private class SavedResults {
        private List<List<AnalizeResult>> analizeResults;
        private String text;
        private String[] listTitles;

        public SavedResults(List<List<AnalizeResult>> analizeResults, String text, String[] listTitles) {
            this.analizeResults = analizeResults;
            this.text = text;
            this.listTitles = listTitles;
        }

        public String[] getListTitles() {
            return listTitles;
        }

        public List<List<AnalizeResult>> getAnalizeResults() {
            return analizeResults;
        }

        public String getText() {
            return text;
        }
    }

    private void drawResults(JPanel resultPanel) {
        SavedResults savedResults = (SavedResults) Memory.get(Memory.SAVED_RESULTS);
        JTextArea text = new JTextArea(savedResults.getText(), 3, 30);
        text.setEditable(false);
        JPanel panelForText = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        panelForText.add(text);
        resultPanel.add(panelForText);

        JPanel detailsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 5));
        JPanel theSymbolPicturePanel = new JPanel();
        theSymbolPicturePanel.setPreferredSize(new Dimension(50, 50));
        theSymbolPicturePanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        detailsPanel.add(theSymbolPicturePanel);
        JList list = new JList(savedResults.getListTitles());
        list.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        list.setVisibleRowCount(4);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        detailsPanel.add(list);
        JPanel persentPanel = new JPanel();
        persentPanel.setLayout(new BoxLayout(persentPanel, BoxLayout.Y_AXIS));
        //detailsPanel.add(persentPanel);
        resultPanel.add(detailsPanel);
        resultPanel.add(persentPanel);
        list.addListSelectionListener(new DrawElementDetails(getDialog(), persentPanel, savedResults.getAnalizeResults(), theSymbolPicturePanel));
        list.setSelectedIndex(0);

        mainPanel.revalidate();
        mainPanel.repaint();
        pack();
    }

    private JDialog getDialog() {
        return this;
    }

    private class DrawElementDetails implements ListSelectionListener {

        private JDialog dialog;
        private JPanel panel;
        private List<List<AnalizeResult>> results;
        private JPanel theSymbolPicturePanel;

        public DrawElementDetails(JDialog dialog, JPanel panel, List<List<AnalizeResult>> results, JPanel theSymbolPicturePanel) {
            this.dialog = dialog;
            this.panel = panel;
            this.results = results;
            this.theSymbolPicturePanel = theSymbolPicturePanel;
        }

        public void valueChanged(ListSelectionEvent e) {
            drawDetails(((JList) e.getSource()).getSelectedIndex());
        }

        public void drawDetails(int index) {
            panel.removeAll();

            theSymbolPicturePanel.removeAll();
            DrawImagePanel imagePanel = new DrawImagePanel(((List<DrawImagePanel>) Memory.get(Memory.IMAGES)).get(index));
            imagePanel.setScale(1);
            theSymbolPicturePanel.add(imagePanel);
            for (AnalizeResult analizeResult : results.get(index)) {
                JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 2, 2));
                panel.add(new JLabel(analizeResult.getLabel() + " : " + String.valueOf(analizeResult.getPersent()) + "%"));
                this.panel.add(panel);
            }
            mainPanel.revalidate();
            mainPanel.repaint();
            dialog.pack();
        }

    }

    private String getTemplateFileName() {
        String result = (String) Memory.get(Memory.SAVING_TEMPLATES);
        return result == null ? "" : result; 
    }

    private class LoadTemplateListener implements ActionListener {

        private JTextField fieldToUpdate;

        public LoadTemplateListener(JTextField fieldToUpdate) {
            this.fieldToUpdate = fieldToUpdate;
        }

        public void actionPerformed(ActionEvent e) {
            LoadFile loadFile = new LoadFile(new TemplateLoader());
            StudiesSet aSet = (StudiesSet) loadFile.load();
            String fileName = loadFile.getFileName();
            Memory.add(Memory.TEMPLATE, aSet);
            Memory.add(Memory.SAVING_TEMPLATES, fileName);
            fieldToUpdate.setText(fileName);
        }
    }

    public void updateLabels(List<Labels> symbols) {
        this.symbols = new LinkedList<Labels>(symbols);
        pack();
    }
}
