package dsuyasov.recognition.ui.helper.dialogs;

import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.ColorGeneratorSelecter;
import dsuyasov.recognition.ui.helper.panels.cellSettingsPanel.SymbolGeneratorSelecter;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.RuleCellResult;
import dsuyasov.recognition.rulesModel.RuleItem;
import dsuyasov.recognition.rulesModel.generators.ColorGenerator;
import dsuyasov.recognition.rulesModel.generators.LabelGenerator;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Dmitriy Suyasov
 *         Date 15.05.2007
 */
public class ResultCellOptionDialog extends AbstractDialog {

    public static final String TITLE = Localization.get(Localization.RESULT_CELL_OPTIONS_DIALOG_TITLE);

    private ColorGeneratorSelecter colorGeneratorSelecter;
    private SymbolGeneratorSelecter symbolGeneratorSelecter;

    private RuleCellResult ruleCellResult;

    public ResultCellOptionDialog() throws HeadlessException {
        
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        setContentPane(mainPanel);

        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        //Integer selectedIndex = (Integer) Memory.get(Memory.SELECTED_RULE);
        RuleItem ruleItem = cellularAutomata.getSelectedRuleItem();
        this.ruleCellResult = ruleItem.getResult();//(selectedIndex == 0) ? cellularAutomata.getDefaultRuleItem().getResult() : cellularAutomata.getRuleItems().get(selectedIndex - 1).getResult();

        colorGeneratorSelecter = new ColorGeneratorSelecter(this, mainPanel);
        colorGeneratorSelecter.initialize(this.ruleCellResult.getColorGenerator());
        symbolGeneratorSelecter = new SymbolGeneratorSelecter(this, mainPanel);
        symbolGeneratorSelecter.initialize(this.ruleCellResult.getLabelGenerator());

        JPanel symbolRemoverPanel = new JPanel();
        symbolRemoverPanel.setLayout(new BoxLayout(symbolRemoverPanel, BoxLayout.Y_AXIS));
        mainPanel.add(symbolRemoverPanel);
        ruleCellResult.getLabelRemover().draw(symbolRemoverPanel, this);

        drawUpdateCancelButtons(mainPanel);
        
        setResizable(false);

        setDefaultCloseOperation(
            JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new CellOptionDialogWindowListener(this));
    }

    public void updateAndClose() {
        ColorGenerator colorGenerator = (ColorGenerator) colorGeneratorSelecter.getResult();
        if (colorGenerator != null) {
            this.ruleCellResult.setColorGenerator(colorGenerator);
        }
        LabelGenerator labelGenerator = (LabelGenerator) symbolGeneratorSelecter.getResult();
        if (labelGenerator != null) {
            this.ruleCellResult.setSymbolGenerator(labelGenerator);
        }
        close();
    }

    public String getDialogTitle() {
        return TITLE;
    }

    private class CellOptionDialogWindowListener extends WindowAdapter {

        private ResultCellOptionDialog parent;

        public CellOptionDialogWindowListener(ResultCellOptionDialog parent) {
            this.parent = parent;
        }

        public void windowClosing(WindowEvent we) {
            int result = JOptionPane.showConfirmDialog(parent, Localization.get(Localization.ACCEPT_CHANGES_QUESTION), Localization.get(Localization.QUESTION_DIALOG_TITLE),
                    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (result == JOptionPane.CANCEL_OPTION)
                return;
            if (result == JOptionPane.YES_OPTION) {
                parent.updateAndClose();
            } else {
                parent.close();
            }
        }
    }

}

