package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.rulesModel.RuleCell;
import dsuyasov.recognition.rulesModel.RuleItem;
import dsuyasov.recognition.ui.helper.dialogs.CellOptionsDialog;
import dsuyasov.recognition.ui.helper.dialogs.ResultCellOptionDialog;
import dsuyasov.recognition.util.DrawUtils;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

/**
 * @author Dmitriy Suyasov
 *         Date 16.04.2007
 */
public class AutomataDescriptionPanel extends JPanel implements MouseListener {

    private JLabel ruleItemLabel;

    public AutomataDescriptionPanel(JPanel parent) {
        parent.setLayout(new BorderLayout(10, 10));

        ruleItemLabel = new JLabel();
        updateRuleItemText();
        
        parent.add(ruleItemLabel, BorderLayout.NORTH);

        Dimension size = new Dimension(500, 500);
        setPreferredSize(size);
        setMinimumSize(size);
        parent.add(this, BorderLayout.CENTER);

        addMouseListener(this);
    }

    public void updateRuleItemText() {
        Integer selectedRuleItem = (Integer) Memory.get(Memory.SELECTED_RULE);
        if (selectedRuleItem == null || selectedRuleItem < 0) {
            ruleItemLabel = new JLabel(Localization.get(Localization.SELECT_RULE_ITEM_TO_SEE_DESCRIPTION));
        } else {
            ruleItemLabel.setText(Localization.get(Localization.RULE_ITEM_TITLE) +
                    (selectedRuleItem == 0 ? Localization.get(Localization.DEFAULT_NAME) : "" + (selectedRuleItem - 1)));
        }
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        Integer selectedRuleItem = (Integer) Memory.get(Memory.SELECTED_RULE);

        if (cellularAutomata == null || selectedRuleItem == null || selectedRuleItem < 0)
            return;

        RuleItem ruleItem = cellularAutomata.getSelectedRuleItem();//(selectedRuleItem == 0) ? cellularAutomata.getDefaultRuleItem() : cellularAutomata.getRuleItems().get(selectedRuleItem - 1);

        int dimention = cellularAutomata.getRuleDimention().getDimention();

        for (int column = -dimention; column <= dimention; column++) {
            for (int line = -dimention; line <= dimention; line++) {
                RuleCell cell = ruleItem.getCell(line, column);
                Rectangle rectangle = new Rectangle(20 + (column + dimention) * 30, 20 + (line + dimention) * 30, 30, 30);
                cell.drawCell(g, rectangle);
            }
        }
        g.setColor(Color.BLACK);

        int baseXPoint = 20 + (dimention * 2 + 1) * 30 + 30;
        int baseYPoint = 20 + dimention * 30;
        
        DrawUtils.drawArrow(g, baseXPoint, baseYPoint + 6, 1.5);
        ruleItem.getResult().drawCell(g, new Rectangle(baseXPoint + 50, baseYPoint, 30, 30));
    }

    public void mouseClicked(MouseEvent e) {
        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        int dimention = cellularAutomata.getRuleDimention().getDimention();
        int length = 30 * (dimention * 2 + 1);
        boolean mainRect = isInsideRectangle(new Rectangle(20, 20, length, length), e.getPoint());
        boolean resultRect = isInsideRectangle(new Rectangle(20 + (dimention * 2 + 1) * 30 + 80,
                20 + dimention * 30, 30, 30), e.getPoint());
        if (mainRect) {
            int column = (e.getX() - 20) / 30 - dimention;
            column = (column > dimention) ? dimention : column;
            int line = (e.getY() - 20) / 30 - dimention;
            line = (line > dimention) ? dimention : line;
            JDialog newRuleSittingsDialog = new CellOptionsDialog(line, column);
            Rectangle appletRect = ((JApplet) Memory.get(Memory.APPLET)).getBounds();
            int dialogX = appletRect.getLocation().x + (appletRect.width / 4);
            int dialogY = appletRect.getLocation().y + (appletRect.height / 4);
            newRuleSittingsDialog.setBounds(new Rectangle(dialogX, dialogY, CellOptionsDialog.WIDTH,
                    CellOptionsDialog.HEIGHT));
            newRuleSittingsDialog.pack();
            newRuleSittingsDialog.setVisible(true);
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
        } else if (resultRect) {
            JDialog newRuleSittingsDialog = new ResultCellOptionDialog();
            Rectangle appletRect = ((JApplet) Memory.get(Memory.APPLET)).getBounds();
            int dialogX = appletRect.getLocation().x + (appletRect.width / 4);
            int dialogY = appletRect.getLocation().y + (appletRect.height / 4);
            newRuleSittingsDialog.setBounds(new Rectangle(dialogX, dialogY, CellOptionsDialog.WIDTH,
                    CellOptionsDialog.HEIGHT));
            newRuleSittingsDialog.pack();
            newRuleSittingsDialog.setVisible(true);
            ((JPanel) Memory.get(Memory.MAIN_PANEL)).repaint();
        }

    }

    private static boolean isInsideRectangle(Rectangle rect, Point point) {
        return point.x >= rect.x && point.x <= rect.x + rect.width &&
                point.y >= rect.y && point.y <= rect.y + rect.height;
    }

    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}

    public void mouseEntered(MouseEvent e) {}

    public void mouseExited(MouseEvent e) {}
}
