package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.rulesModel.CellularAutomata;
import dsuyasov.recognition.rulesModel.RuleItem;
import dsuyasov.recognition.rulesModel.RuleCell;
import dsuyasov.recognition.util.MessageUtil;
import dsuyasov.recognition.util.DrawUtils;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

/**
 * @author Dmitriy Suyasov
 */
public class AutomataSelectionPanel extends JPanel implements MouseListener {

    private int ruleHeight;

    public AutomataSelectionPanel() {
        addMouseListener(this);
        Memory.add(Memory.SELECTED_RULE, 0);
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        g.drawString(Localization.get(Localization.DEFAULT_TITLE), 10, 20);
        int dimention = cellularAutomata.getRuleDimention().getDimention();
        this.ruleHeight = (dimention * 2 + 1) * 10;
        int rulesBasePoint = 50 + ruleHeight;
        g.drawString(Localization.get(Localization.RULES_TITLE), 10, rulesBasePoint);

        if ((Integer) Memory.get(Memory.SELECTED_RULE) == 0) {
            drawSelection(g, 25);
        }
        drawRuleItem(cellularAutomata.getDefaultRuleItem(), dimention, g, 10, 30);

        for (int iLoop = 0; iLoop < cellularAutomata.getRuleItems().size(); iLoop++) {
            RuleItem ruleItem = cellularAutomata.getRuleItems().get(iLoop);
            int ruleItemY = rulesBasePoint + 10 + iLoop * (ruleHeight + 10);
            if (iLoop + 1 == (Integer) Memory.get(Memory.SELECTED_RULE)) {
                drawSelection(g, ruleItemY - 5);
            }
            drawRuleItem(ruleItem, dimention, g, 10, ruleItemY);
        }
    }

    private void drawSelection(Graphics g, int y) {
        g.setColor(Color.getHSBColor(0.5f, 0.5f, 0.7f));
        g.fillRect(0, y, 130, ruleHeight + 10);
    }

    private void drawRuleItem(RuleItem ruleItem, int dimention, Graphics g, int x, int y) {
        for (int line = -dimention; line <= dimention; line++) {
            for (int column = -dimention; column <= dimention; column++) {
                RuleCell cell = ruleItem.getCell(line, column);
                Rectangle rectangle = new Rectangle(x + (column + dimention) * 10, y + (line + dimention) * 10, 10, 10);
                cell.drawCell(g, rectangle);
                //DrawUtils.drawCell(g, cell, rectangle);
            }
        }
        g.setColor(Color.BLACK);

        int baseXPoint = x + (dimention * 2 + 1) * 10 + 15;
        int baseYPoint = y + dimention * 10;
        DrawUtils.drawArrow(g, baseXPoint, baseYPoint, 1);

        Rectangle resultRectangle = new Rectangle(baseXPoint + 30, baseYPoint, 10, 10);
        ruleItem.getResult().drawCell(g, resultRectangle);
    }

    public Rectangle getSizes() {
        CellularAutomata cellularAutomata = (CellularAutomata) Memory.get(Memory.CURRENT_RULE);
        int height = (1 + cellularAutomata.getRuleItems().size()) * (10 * (cellularAutomata.getRuleDimention().getDimention() * 2 + 1) + 10) + 55;
        return new Rectangle(0, 0, 130, height);
    }

    public void mouseClicked(MouseEvent e) {
        int ruleItemsCount = ((CellularAutomata) Memory.get(Memory.CURRENT_RULE)).getRuleItems().size();
        int mouseY = e.getY();
        int selectedRule = (Integer) Memory.get(Memory.SELECTED_RULE);
        int newSelectedRule = (mouseY >= 25 && mouseY <= ruleHeight + 35) ? 0 :
                (mouseY >= ruleHeight + 55 && mouseY <= ruleHeight + 55 + ruleItemsCount * (ruleHeight + 10)) ?
                        1 + (mouseY - ruleHeight - 55) / (ruleHeight + 10) : selectedRule;
        if (newSelectedRule != selectedRule) {
            Memory.add(Memory.SELECTED_RULE, newSelectedRule);
            MessageUtil.setMessage(newSelectedRule == 0 ? Localization.get(Localization.DEFAULT_RULE_SELECTED_MESSAGE) :
                    (Localization.get(Localization.RULE_TITLE) + newSelectedRule + " " +
                            Localization.get(Localization.SELECTED_TITLE)));
            ((JPanel) Memory.get(Memory.RULE_ITEM_PANEL)).repaint();
        }
        this.repaint();
    }

    public void mousePressed(MouseEvent e) {

    }

    public void mouseReleased(MouseEvent e) {

    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {

    }
}
