package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.rulesModel.sequence.SequenceDisplayable;
import dsuyasov.recognition.rulesModel.Memory;

import javax.swing.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.List;

/**
 * @author Dmitriy Suyasov
 *         Date 17.05.2007
 */
public class AutomataSequencePanel extends JPanel {

    public static final int Y_RULES_START_POINT = 15;
    public static final int X_RULES_START_POINT = 5;
    public static final int RULES_STEP = 40;
    public static final int RULES_HEIGHT = 30;
    public static final int RULES_WIDTH = 150;
    public static final int Y_TEXT_OFFSET = RULES_HEIGHT - 10;
    public static final int X_TEXT_OFFSET = 5;

    private AutomataSequence automataSequence;
    private boolean current;
    private boolean indicateAutomata;

    public AutomataSequencePanel() {
        this.automataSequence = AutomataSequence.getRulesSequence();
        current = true;
    }

    public AutomataSequencePanel(AutomataSequence automataSequence, boolean indicateAutomata) {
        this.automataSequence = automataSequence;
        this.current = false;
        this.indicateAutomata = indicateAutomata;
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        if (current)
            this.automataSequence = AutomataSequence.getRulesSequence();

        g.setColor(Color.BLACK);
        int y = Y_RULES_START_POINT;
        for (int iLoop = 0; iLoop < automataSequence.getRules().size(); iLoop++) {
            SequenceDisplayable sequenceDisplayable = automataSequence.getRules().get(iLoop);

            if (indicateAutomata) {
                List<AutomataSequence.ForProcess> list = (List<AutomataSequence.ForProcess>) Memory.get(Memory.PROCESSING_ELEMENTS);
                if (list != null && !list.isEmpty() && iLoop == list.get(0).getNext()) {
                    g.setColor(Color.WHITE);
                    g.fillRect(1, y - 3, RULES_WIDTH + 8, RULES_HEIGHT + 6);
                    g.setColor(Color.BLACK);
                    g.drawRect(1, y - 3, RULES_WIDTH + 8, RULES_HEIGHT + 6);
                }
            }

            g.setColor(Color.BLACK);

            paintForRule(g, iLoop);

            sequenceDisplayable.draw(g, new Rectangle(X_RULES_START_POINT, y, RULES_WIDTH, RULES_HEIGHT), X_TEXT_OFFSET, Y_TEXT_OFFSET);
            if (y > Y_RULES_START_POINT) {
                g.fillOval(X_RULES_START_POINT - 3 + RULES_WIDTH / 2, y - 3, 6, 6);
                g.fillOval(X_RULES_START_POINT - 3 + RULES_WIDTH / 2, y - (RULES_STEP - RULES_HEIGHT) - 3, 6, 6);
                g.drawLine(X_RULES_START_POINT + RULES_WIDTH / 2, y, X_RULES_START_POINT + RULES_WIDTH / 2, y - (RULES_STEP - RULES_HEIGHT));
            }
            y += RULES_STEP;
        }
    }

    protected void paintForRule(Graphics g, int iLoop) {}

    public Rectangle getSizes() {
        if (current)
            this.automataSequence = AutomataSequence.getRulesSequence();
        int height = 40 + RULES_STEP * automataSequence.getRules().size();
        return new Rectangle(0, 0, RULES_WIDTH + X_RULES_START_POINT * 2, height);
    }

}
