package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.sequence.AutomataSequence;
import dsuyasov.recognition.rulesModel.sequence.SequenceDisplayable;
import dsuyasov.recognition.rulesModel.sequence.SpecialElement;
import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.ui.helper.dialogs.ChooseSpecialRuleDialog;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;

/**
 * @author Dmitriy Suyasov
 */
public class AutomataSequenceWithSettingsPanel extends AutomataSequencePanel {

    private int selectedRule;
    private JTextArea ruleDescription;

    public AutomataSequenceWithSettingsPanel() {
        addMouseListener(new PopUpListener(this));
        this.selectedRule = -1;
    }

    public void setRuleDescription(JTextArea ruleDescription) {
        this.ruleDescription = ruleDescription;
    }

    public int getSelectedRule() {
        return selectedRule;
    }

    protected void paintForRule(Graphics g, int iLoop) {
        if (iLoop == selectedRule) {
            g.setColor(Color.LIGHT_GRAY);
            g.fillRect(X_RULES_START_POINT, Y_RULES_START_POINT + iLoop * RULES_STEP, RULES_WIDTH, RULES_HEIGHT);
            g.setColor(Color.BLACK);
        }
    }

    public void updateSelection(int offset) {
        int size = AutomataSequence.getRulesSequence().getRules().size();
        if (size <= selectedRule + offset)
            selectedRule = size - 1;
        else if (selectedRule + offset < 0)
            selectedRule = 0;
        else
            selectedRule += offset;
    }

    private class PopUpListener extends MouseAdapter {

        JPanel parentPanel;
        JPopupMenu popupMenu;

        public PopUpListener(JPanel parentPanel) {
            this.parentPanel = parentPanel;
        }

        public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {

            if (popupMenu != null)
                parentPanel.remove(popupMenu);

            if (e.isPopupTrigger()) {
                for (int iLoop = 0; iLoop < AutomataSequence.getRulesSequence().getRules().size(); iLoop++) {
                    SequenceDisplayable sequenceDisplayable = AutomataSequence.getRulesSequence().getRules().get(iLoop);
                    if (new Rectangle(X_RULES_START_POINT, Y_RULES_START_POINT + iLoop * RULES_STEP, RULES_WIDTH, RULES_HEIGHT).contains(e.getX(), e.getY())) {
                        popupMenu = new JPopupMenu();

                        if (sequenceDisplayable.isRule()) {
                            JMenuItem addAsCurrentRuleMenuItem = new JMenuItem(Localization.get(Localization.ADD_AS_CURRENT_RULE_MENU_ITEM_NAME));
                            popupMenu.add(addAsCurrentRuleMenuItem);
                            addAsCurrentRuleMenuItem.addActionListener(new AddAsCurrentRuleListener(iLoop));
                        }
                        if (sequenceDisplayable.isSpecialElement()) {
                            JMenuItem specialRuleSettingsMenuItem = new JMenuItem(Localization.get(Localization.SPECIAL_SYMBOL_SETTINGS_MENU_ITEM_NAME));
                            popupMenu.add(specialRuleSettingsMenuItem);
                            specialRuleSettingsMenuItem.addActionListener(new SpecialRuleSettingsListener(iLoop));
                        }

                        parentPanel.add(popupMenu);
                        popupMenu.show(e.getComponent(), e.getX(), e.getY());

                        break;
                    }
                }
            }
        }

        public void mouseClicked(MouseEvent e) {
            int mouseX = e.getX();
            int mouseY = e.getY();
            int ruleSize = AutomataSequence.getRulesSequence().getRules().size();
            if (mouseX >= X_RULES_START_POINT && mouseX <= X_RULES_START_POINT + RULES_WIDTH &&
                    mouseY >= Y_RULES_START_POINT && mouseY <= Y_RULES_START_POINT + ruleSize * RULES_STEP &&
                    (mouseY - Y_RULES_START_POINT) % RULES_STEP <= RULES_HEIGHT) {
                selectedRule = (mouseY - Y_RULES_START_POINT) / RULES_STEP;
                if (ruleDescription != null)
                    ruleDescription.setText(Localization.get(Localization.CELLULAR_AUTOMATA_DESCRIPTION_MESSAGE)+": " +
                            AutomataSequence.getRulesSequence().getRules().get(selectedRule).getDescription());
                repaint();
            }
        }
    }

    private class AddAsCurrentRuleListener implements ActionListener {

        private int ruleSequenceIndex;

        public AddAsCurrentRuleListener(int ruleSequenceIndex) {
            this.ruleSequenceIndex = ruleSequenceIndex;
        }

        public void actionPerformed(ActionEvent e) {
            Memory.add(Memory.CURRENT_RULE, AutomataSequence.getRulesSequence().getRules().get(ruleSequenceIndex));
        }
    }

    private class SpecialRuleSettingsListener implements ActionListener {

        private int ruleSequenceIndex;

        public SpecialRuleSettingsListener(int ruleSequenceIndex) {
            this.ruleSequenceIndex = ruleSequenceIndex;
        }

        public void actionPerformed(ActionEvent e) {
            new ChooseSpecialRuleDialog((SpecialElement) AutomataSequence.getRulesSequence().getRules().get(ruleSequenceIndex));
            scrollRectToVisible(getSizes());
            setPreferredSize(getSizes().getSize());
            revalidate();
            repaint();
        }
    }

}
