package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.MatrixElement;
import dsuyasov.recognition.rulesModel.labels.Labels;

import javax.swing.*;
import java.awt.*;
import java.util.List;

/**
 * @author Dmitriy Suyasov
 */
public class DrawImagePanel extends JPanel {

    private MatrixElement[][] elements;
    private int scale;
    private List<Labels> symbols;

    public DrawImagePanel(MatrixElement[][] elements, List<Labels> symbols) {
        super(true);
        this.elements = elements;
        this.scale = 1;
        this.symbols = symbols;
    }

    public DrawImagePanel(DrawImagePanel anotherPanel) {
        super(true);
        this.elements = anotherPanel.getElements();
        this.scale = anotherPanel.getScale();
        this.symbols = anotherPanel.getSymbols();
    }

    public MatrixElement[][] getElements() {
        return elements;
    }

    public int getScale() {
        return scale;
    }

    public List<Labels> getSymbols() {
        return symbols;
    }

    public void paint(Graphics g) {
        super.paint(g);
        int width= elements.length;
        int height = elements[0].length;
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                g.setColor(elements[x][y].getColor());
                g.fillRect(x*scale, y*scale, scale, scale);

                if (scale >= 8) {
                    int offset = 1;
                    for (int iLoop = 0; iLoop < symbols.size(); iLoop++) {
                        Labels symbol = symbols.get(iLoop);
                        if (elements[x][y].contain(symbol)) {
                            int width1 = (scale - 2) / symbols.size();
                            width1 = width1 == 0 ? 1 : width1;
                            symbol.draw(g, x*scale+offset - width1 / 2, y*scale + 1 - width1 / 2, width1 * 2, width1 * 2);
                            offset += width1;
                        }
                    }
                }
            }
        }
    }

    public int getWidth() {
        return elements.length * scale;
    }

    public int getHeight() {
        return elements[0].length * scale;
    }

    public void setScale(int scale) {
        this.scale = scale;
    }
}