package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.util.MessageUtil;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;

/**
 * @author Dmitriy Suyasov
 */
public class ImagePanel extends JPanel implements MouseMotionListener, MouseListener {

    public double SCALE = 1d;

    private BufferedImage image;
    private boolean loadedImage;

    private SliderChangeListener listener;

    public ImagePanel(LayoutManager layoutManager) {
        super(layoutManager);
        this.image = ((BufferedImage) Memory.get(Memory.PICTURE));
        this.loadedImage = true;
        init();
    }

    private void init() {
        listener = new SliderChangeListener(this);

        setSize(image.getWidth(), image.getHeight());
        setAutoscrolls(true);

        addMouseMotionListener(this);
        addMouseListener(this);
    }

    public SliderChangeListener getListener() {
        return listener;
    }

    public boolean isLoadedImage() {
        return loadedImage;
    }

    public BufferedImage getImage() {
        return image;
    }

    public void paint(Graphics graphics) {
        super.paint(graphics);
        BufferedImage bufimage = loadedImage ? (BufferedImage) Memory.get(Memory.PICTURE) : image;
        graphics.drawImage(bufimage, 0, 0, (int) Math.round(bufimage.getWidth() / SCALE), (int) Math.round(bufimage.getHeight() / SCALE), this);
    }

    public void mouseDragged(MouseEvent e) {
        Rectangle r = new Rectangle(e.getX(), e.getY(), 1, 1);
        scrollRectToVisible(r);
    }

    public void mouseMoved(MouseEvent mouseEvent) {
        MessageUtil.setMessage("X: " + (Math.round((mouseEvent.getX())* SCALE)) +
            " Y: " + (Math.round((mouseEvent.getY())* SCALE)));
    }

    public void mouseClicked(MouseEvent e) {

    }

    public void mousePressed(MouseEvent e) {

    }

    public void mouseReleased(MouseEvent e) {

    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {

    }

    public class SliderChangeListener implements ChangeListener {

        private ImagePanel imagePanel;

        public SliderChangeListener(ImagePanel imagePanel) {
            this.imagePanel = imagePanel;
        }

        public void stateChanged(ChangeEvent e) {
            ImagePanel panel = imagePanel;

            JSlider slider = (JSlider)e.getSource();
            double value = slider.getValue();
            value = value / 100;
            panel.SCALE = (value > 0) ? (1/(value + 1)) : -(value - 1);

            BufferedImage bufimage = imagePanel.isLoadedImage() ? (BufferedImage) Memory.get(Memory.PICTURE) : imagePanel.getImage();

            Rectangle rect = new Rectangle((int) Math.round(bufimage.getWidth() / panel.SCALE), (int) Math.round(bufimage.getHeight() / panel.SCALE));
            Dimension dim = new Dimension((int) Math.round(bufimage.getWidth() / panel.SCALE), (int) Math.round(bufimage.getHeight() / panel.SCALE));
            panel.setSize(dim);
            panel.scrollRectToVisible(rect);
            panel.setPreferredSize(dim);

            panel.revalidate();
            panel.repaint();
        }

    }
}
