package dsuyasov.recognition.ui.helper.panels;

import dsuyasov.recognition.rulesModel.labels.Labels;
import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.ui.helper.dialogs.LabelsOptionDialog;
import dsuyasov.recognition.ui.helper.LabelsChanges;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.LinkedList;

/**
 * @author Dmitriy Suyasov
 *         Date 18.05.2007
 */
public class SelectLabelsPanel extends JPanel {

    private List<Labels> defaultLabels;
    private String memoryLabel;

    public SelectLabelsPanel(List<Labels> defaultLabels, LabelsChanges labelsChanges) {
        super(new FlowLayout(FlowLayout.LEFT, 5, 5));

        this.defaultLabels = new LinkedList<Labels>(defaultLabels);
        this.memoryLabel = null;

        JPanel labelsPanel = new LabelsPanel();
        Dimension preferredSize = new Dimension(defaultLabels.size() * 32, 30);
        labelsPanel.setPreferredSize(preferredSize);
        labelsPanel.setSize(preferredSize);
        add(labelsPanel);

        JButton changeLabelsButton = new JButton(Localization.get(Localization.CHANGE_BUTTON_NAME));
        changeLabelsButton.addActionListener(new ChangeLabelsListener(labelsChanges, labelsPanel));
        add(changeLabelsButton);
    }

    public SelectLabelsPanel(List<Labels> defaultLabels, LabelsChanges labelsChanges, String memoryLabel) {
        this(defaultLabels, labelsChanges);
        this.memoryLabel = memoryLabel;
    }

    private class LabelsPanel extends JPanel {
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            int x = 1;
            for (Labels labels : defaultLabels) {
                g.setColor(Color.BLACK);
                g.drawRect(x - 1, 0, getWidth() / defaultLabels.size() - 2, getHeight() - 1);
                labels.draw(g, x, 1, getWidth() / defaultLabels.size() - 3, getHeight() - 2);
                x += getWidth() / defaultLabels.size();
            }
        }
    }

    private class ChangeLabelsListener implements ActionListener {

        private LabelsChanges labelsChanges;
        private JPanel LabelsPanel;

        public ChangeLabelsListener(LabelsChanges labelsChanges, JPanel labelsPanel) {
            this.labelsChanges = labelsChanges;
            this.LabelsPanel = labelsPanel;
        }

        public void actionPerformed(ActionEvent e) {
            LabelsOptionDialog dialog = new LabelsOptionDialog(defaultLabels, false);

            dialog.pack();
            dialog.setVisible(true);

            if (dialog.getLabels() != null) {
                defaultLabels = dialog.getLabels();
                if (memoryLabel != null) {
                    Memory.add(memoryLabel, defaultLabels);
                }
                Dimension preferredSize = new Dimension(defaultLabels.size() * 32, 30);
                LabelsPanel.setPreferredSize(preferredSize);
                LabelsPanel.setSize(preferredSize);
                LabelsPanel.repaint();

                labelsChanges.updateLabels(defaultLabels);
            }
        }

    }
}
