package dsuyasov.recognition.ui.page;

import dsuyasov.recognition.rulesModel.Memory;
import dsuyasov.recognition.util.localization.Localization;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author Dmitriy Suyasov
 *         Date 09.04.2007
 */
public abstract class Page {

    public static final String NEXT = Localization.get(Localization.NEXT_PAGE_BUTTON_NAME);
    public static final String PREVIOUS = Localization.get(Localization.PREVIOUS_PAGE_BUTTON_NAME);

    private Page previous;
    private Page next;

    protected Page() {
        previous = null;
        next = null;
        Memory.add(getUniqueKey(), this);
    }

    protected Page(Page previous, Page next) {
        this.previous = previous;
        this.next = next;
        Memory.add(getUniqueKey(), this);
    }

    public Page getPrevious() {
        return previous;
    }

    public void setPrevious(Page previous) {
        this.previous = previous;
    }

    public Page getNext() {
        return next;
    }

    public void setNext(Page next) {
        this.next = next;
        next.setPrevious(this);
    }

    public void drawAll() {
        Memory.add(Memory.CURRENT_PAGE, this);
        drawDescription();
        draw();
        drawButtons();
        ((JPanel) Memory.get(Memory.ROOT_PANEL)).setVisible(false);
        ((JPanel) Memory.get(Memory.ROOT_PANEL)).setVisible(true);
    }

    public abstract String getUniqueKey();

    public abstract void draw();

    private void drawButtons() {
        JPanel buttonsPanel = (JPanel) Memory.get(Memory.BUTTONS_PANEL);

        JPanel layoutPanel = new JPanel();
        layoutPanel.setLayout(new BoxLayout(layoutPanel, BoxLayout.LINE_AXIS));
        layoutPanel.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);


        if (next != null) {
            JButton nextButton = new JButton(NEXT);
            nextButton.setSize(200, 20);
            nextButton.addActionListener(new GoNextPageListener(this, true));
            layoutPanel.add(nextButton);
        }
        if (previous != null) {
            JButton previousButton = new JButton(PREVIOUS);
            previousButton.setSize(200, 20);
            previousButton.addActionListener(new GoNextPageListener(this, false));
            layoutPanel.add(previousButton);
        }

        buttonsPanel.removeAll();
        buttonsPanel.add(layoutPanel);
    }

    private void drawDescription() {
        JPanel descriptionPanel = (JPanel) Memory.get(Memory.DESCRIPTION_PANEL);

        JPanel descriptionLabelPanel = new JPanel();
        JLabel descriptionLabel = new JLabel(getDescription());
        descriptionLabel.setFont(new Font("Arial", Font.BOLD, 14));
        descriptionLabel.setSize(descriptionPanel.getSize());
        descriptionLabelPanel.add(descriptionLabel);

        descriptionPanel.removeAll();
        descriptionPanel.add(descriptionLabelPanel);
    }

    public abstract String getDescription();

    public static void setAsCurrent(String name) {
        Page page = (Page) Memory.get(name);
        if (page != null) {
            page.drawAll();
        }
    }

    private static class GoNextPageListener implements ActionListener {

        private Page helper;
        private boolean isNext;

        public GoNextPageListener(Page helper, boolean isNext) {
            this.helper = helper;
            this.isNext = isNext;
        }

        public void actionPerformed(ActionEvent e) {
            if (isNext) {
                if (helper.getNext() != null) {
                    helper.getNext().drawAll();
                }
            } else {
                if (helper.getPrevious() != null) {
                    helper.getPrevious().drawAll();
                }
            }
        }
    }

    public static void initialize(JApplet applet) {
        JPanel rootPanel = new JPanel(new BorderLayout(10, 10));

        JPanel descriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 10, 10));
        rootPanel.add(descriptionPanel, BorderLayout.NORTH);

        JPanel mainPanel = new JPanel(new GridLayout(1,1));
        rootPanel.add(mainPanel, BorderLayout.CENTER);
        
        JPanel downPanels = new JPanel(new BorderLayout(2, 2));
        rootPanel.add(downPanels, BorderLayout.SOUTH);
        JPanel buttonsPanel = new JPanel(new GridLayout(1,1));
        buttonsPanel.setMinimumSize(new Dimension(500, 200));
        buttonsPanel.setMaximumSize(new Dimension(3000, 200));
        downPanels.add(buttonsPanel, BorderLayout.NORTH);

        JLabel messageLabel = new JLabel(" ");
        messageLabel.setBorder(BorderFactory.createLoweredBevelBorder());
        downPanels.add(messageLabel, BorderLayout.SOUTH);

        applet.getContentPane().add(rootPanel);

        Page panel1 = new ShowPicturePage();
        Page panel2 = new ShowAutomataPage();
        Page panel3 = new CombineAutomataPage();
        Page panel4 = new StartAutomataSequencePage();
        panel1.setNext(panel2);
        panel2.setNext(panel3);
        panel3.setNext(panel4);
        panel4.setNext(new ResolvingPage());
        Memory.add(Memory.CURRENT_PAGE, panel1);

        Memory.add(Memory.APPLET, applet);
        Memory.add(Memory.ROOT_PANEL, rootPanel);
        Memory.add(Memory.MAIN_PANEL, mainPanel);
        Memory.add(Memory.DESCRIPTION_PANEL, descriptionPanel);
        Memory.add(Memory.BUTTONS_PANEL, buttonsPanel);
        Memory.add(Memory.MESSAGE_LABEL, messageLabel);

        panel1.drawAll();

        applet.repaint();
    }

}
